# -*- coding: utf-8 -*-
import numpy as np
import logging
import matplotlib.pyplot as plt

# pyCGM2
import pyCGM2
from pyCGM2 import enums
from pyCGM2.Report import plot, plotViewers






class KinematicsPlotComparisonViewer(plotViewers.AbstractPlotViewer):
    """


    """

    def __init__(self,iAnalyses,context,legends,pointLabelSuffix_lst=None,bodyPart=enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `iAnalyses` (list ) - list of ` of pyCGM2.Processing.analysis.Analysis`
                 - `context` (str ) - list of ` of pyCGM2.Processing.analysis.Analysis`
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
    """


        super(KinematicsPlotComparisonViewer, self).__init__(iAnalyses)


        for itAnalysis in iAnalyses:
            if isinstance(itAnalysis,pyCGM2.Processing.analysis.Analysis):
                pass
            else:
                logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

        self.m_analysis = self.m_input
        self.m_context = context
        self.m_pointLabelSuffixes = pointLabelSuffix_lst
        self.m_normativeData = None
        self.m_flagConsistencyOnly = False
        self.m_legends = legends
        self.m_bodyPart = bodyPart

        if len(iAnalyses) != len(legends):
            raise Exception("legends don t match analysis. Must have same length")
        if pointLabelSuffix_lst is not None:
            if len(iAnalyses) != len(pointLabelSuffix_lst):
                raise Exception("list of point label suffix don t match analysis. Must have same length")

        self.m_concretePlotFunction = None

    def setConcretePlotFunction(self, concreteplotFunction):
        self.m_concretePlotFunction = concreteplotFunction

    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")

        if self.m_concretePlotFunction.func_name in ["descriptivePlot","gaitDescriptivePlot"]:
            title=u""" Descriptive Time-normalized Kinematics \n """
        elif self.m_concretePlotFunction.func_name in ["consistencyPlot","gaitConsistencyPlot"]:
            title=u""" Consistency Time-normalized Kinematics \n """
        else :
            title=u"""\n"""

        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(6,3,1)# Pelvis X
        ax1 = plt.subplot(6,3,2)# Pelvis Y
        ax2 = plt.subplot(6,3,3)# Pelvis Z
        ax3 = plt.subplot(6,3,4)# Hip X
        ax4 = plt.subplot(6,3,5)# Hip Y
        ax5 = plt.subplot(6,3,6)# Hip Z
        ax6 = plt.subplot(6,3,7)# Knee X
        ax7 = plt.subplot(6,3,8)# Knee Y
        ax8 = plt.subplot(6,3,9)# Knee Z
        ax9 = plt.subplot(6,3,10)# Ankle X
        ax10 = plt.subplot(6,3,11)# Ankle Z
        ax11 = plt.subplot(6,3,12)# Footprogress Z
        ax12 = plt.subplot(6,3,13)# ForeFoot X
        ax13 = plt.subplot(6,3,14)# ForeFoot Z
        ax14 = plt.subplot(6,3,15)# ForeFoot Y

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            ax0.set_title("Pelvis Tilt" ,size=8)
            ax1.set_title("Pelvis Obliquity" ,size=8)
            ax2.set_title("Pelvis Rotation" ,size=8)
            ax3.set_title("Hip Flexion" ,size=8)
            ax4.set_title("Hip Adduction" ,size=8)
            ax5.set_title("Hip Rotation" ,size=8)
            ax6.set_title("Knee Flexion" ,size=8)
            ax7.set_title("Knee Adduction" ,size=8)
            ax8.set_title("Knee Rotation" ,size=8)
            ax9.set_title("Ankle dorsiflexion" ,size=8)
            ax10.set_title("Ankle supination" ,size=8)
            ax11.set_title("Foot Progression " ,size=8)
            ax12.set_title("ForeFoot dorsiflexion " ,size=8)
            ax13.set_title("ForeFoot eversion " ,size=8)
            ax14.set_title("ForeFoot Adduction " ,size=8)

            for ax in self.fig.axes:
                ax.set_ylabel("angle (deg)",size=8)

            ax12.set_xlabel("Cycle %",size=8)
            ax13.set_xlabel("Cycle %",size=8)
            ax14.set_xlabel("Cycle %",size=8)

            if not self.m_automaticYlim_flag:
                ax0.set_ylim([0,60])
                ax1.set_ylim([-30,30])
                ax2.set_ylim([-30,30])

                ax3.set_ylim( [-20,70])
                ax4.set_ylim([-30,30])
                ax5.set_ylim([-30,30])

                ax6.set_ylim([-15,75])
                ax7.set_ylim([-30,30])
                ax8.set_ylim([-30,30])

                ax9.set_ylim([-30,30])
                ax10.set_ylim([-30,30])
                ax11.set_ylim([-30,30])

                ax12.set_ylim([-50,30])
                ax13.set_ylim([-30,30])
                ax14.set_ylim([-30,30])
        else:
            raise Exception ("Plot Panel not implemented yet")


    def __setLegend(self,axisIndex):
        self.fig.axes[axisIndex].legend(fontsize=6)
        #self.fig.axes[axisIndex].legend(fontsize=6, bbox_to_anchor=(0,1.2,1,0.2), loc="lower left",
        #    mode="None", borderaxespad=0, ncol=len(self.m_analysis))

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        # iNormativeDataSet.constructNormativeData()
        self.m_normativeData = iNormativeDataSet.data

    def __setData(self):


        if self.m_context == "Left":
            colormap = plt.cm.Reds
            colormap_i=[colormap(i) for i in np.linspace(0.2, 1, len(self.m_analysis))]

            i = 0
            for analysis in self.m_analysis:

                if self.m_pointLabelSuffixes is not None:
                    suffixPlus = "_" + self.m_pointLabelSuffixes[i] if self.m_pointLabelSuffixes[i] !="" else ""
                else:
                    suffixPlus=""

                legend= self.m_legends[i]

                if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                    self.m_concretePlotFunction(self.fig.axes[0],analysis.kinematicStats,
                            "LPelvisAngles"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None,
                    legendLabel=legend)

                    self.m_concretePlotFunction(self.fig.axes[1],analysis.kinematicStats,
                            "LPelvisAngles"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[2],analysis.kinematicStats,
                            "LPelvisAngles"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    # hip
                    self.m_concretePlotFunction(self.fig.axes[3],analysis.kinematicStats,
                            "LHipAngles"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[4],analysis.kinematicStats,
                            "LHipAngles"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[5],analysis.kinematicStats,
                            "LHipAngles"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    # knee
                    self.m_concretePlotFunction(self.fig.axes[6],analysis.kinematicStats,
                            "LKneeAngles"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[7],analysis.kinematicStats,
                            "LKneeAngles"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[8],analysis.kinematicStats,
                            "LKneeAngles"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    # ankle
                    self.m_concretePlotFunction(self.fig.axes[9],analysis.kinematicStats,
                            "LAnkleAngles"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[10],analysis.kinematicStats,
                            "LAnkleAngles"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)

                    # foot progress
                    self.m_concretePlotFunction(self.fig.axes[11],analysis.kinematicStats,
                            "LFootProgressAngles"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    #forefoot
                    self.m_concretePlotFunction(self.fig.axes[12],analysis.kinematicStats,
                            "LForeFootAngles"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[13],analysis.kinematicStats,
                            "LForeFootAngles"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[14],analysis.kinematicStats,
                            "LForeFootAngles"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                i+=1

        if self.m_context == "Right":
            colormap = plt.cm.Blues
            colormap_i=[colormap(i) for i in np.linspace(0.2, 1, len(self.m_analysis))]

            i = 0
            for analysis in self.m_analysis:

                if self.m_pointLabelSuffixes is not None:
                    suffixPlus = "_" + self.m_pointLabelSuffixes[i] if self.m_pointLabelSuffixes[i] !="" else ""
                else:
                    suffixPlus=""

                legend= self.m_legends[i]

                if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                    self.m_concretePlotFunction(self.fig.axes[0],analysis.kinematicStats,
                            "RPelvisAngles"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None,
                    legendLabel=legend)


                    self.m_concretePlotFunction(self.fig.axes[1],analysis.kinematicStats,
                            "RPelvisAngles"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[2],analysis.kinematicStats,
                            "RPelvisAngles"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    # hip
                    self.m_concretePlotFunction(self.fig.axes[3],analysis.kinematicStats,
                            "RHipAngles"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[4],analysis.kinematicStats,
                            "RHipAngles"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[5],analysis.kinematicStats,
                            "RHipAngles"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    # knee
                    self.m_concretePlotFunction(self.fig.axes[6],analysis.kinematicStats,
                            "RKneeAngles"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[7],analysis.kinematicStats,
                            "RKneeAngles"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[8],analysis.kinematicStats,
                            "RKneeAngles"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    # ankle
                    self.m_concretePlotFunction(self.fig.axes[9],analysis.kinematicStats,
                            "RAnkleAngles"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[10],analysis.kinematicStats,
                            "RAnkleAngles"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)

                    # foot progress
                    self.m_concretePlotFunction(self.fig.axes[11],analysis.kinematicStats,
                            "RFootProgressAngles"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)
            #
                    # foreFoot
                    self.m_concretePlotFunction(self.fig.axes[12],analysis.kinematicStats,
                            "RForeFootAngles"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[13],analysis.kinematicStats,
                            "RForeFootAngles"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)
                    self.m_concretePlotFunction(self.fig.axes[14],analysis.kinematicStats,
                            "RForeFootAngles"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                i+=1

    def plotPanel(self):

        if self.m_concretePlotFunction is None:
            raise Exception ("[pyCGM2] need definition of the concrete plot function")


        self.__setLayer()
        self.__setData()
        self.__setLegend(0)

        if self.m_normativeData is not None:
            if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                self.fig.axes[0].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["PelvisAngles"]["mean"][:,0]-self.m_normativeData["PelvisAngles"]["sd"][:,0],
                    self.m_normativeData["PelvisAngles"]["mean"][:,0]+self.m_normativeData["PelvisAngles"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[1].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["PelvisAngles"]["mean"][:,1]-self.m_normativeData["PelvisAngles"]["sd"][:,1],
                    self.m_normativeData["PelvisAngles"]["mean"][:,1]+self.m_normativeData["PelvisAngles"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[2].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["PelvisAngles"]["mean"][:,2]-self.m_normativeData["PelvisAngles"]["sd"][:,2],
                    self.m_normativeData["PelvisAngles"]["mean"][:,2]+self.m_normativeData["PelvisAngles"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)


                self.fig.axes[3].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipAngles"]["mean"][:,0]-self.m_normativeData["HipAngles"]["sd"][:,0],
                    self.m_normativeData["HipAngles"]["mean"][:,0]+self.m_normativeData["HipAngles"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[4].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipAngles"]["mean"][:,1]-self.m_normativeData["HipAngles"]["sd"][:,1],
                    self.m_normativeData["HipAngles"]["mean"][:,1]+self.m_normativeData["HipAngles"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[5].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipAngles"]["mean"][:,2]-self.m_normativeData["HipAngles"]["sd"][:,2],
                    self.m_normativeData["HipAngles"]["mean"][:,2]+self.m_normativeData["HipAngles"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[6].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["KneeAngles"]["mean"][:,0]-self.m_normativeData["KneeAngles"]["sd"][:,0],
                    self.m_normativeData["KneeAngles"]["mean"][:,0]+self.m_normativeData["KneeAngles"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[9].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["AnkleAngles"]["mean"][:,0]-self.m_normativeData["AnkleAngles"]["sd"][:,0],
                    self.m_normativeData["AnkleAngles"]["mean"][:,0]+self.m_normativeData["AnkleAngles"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[11].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["FootAngles"]["mean"][:,2]-self.m_normativeData["FootProgressAngles"]["sd"][:,2],
                    self.m_normativeData["FootAngles"]["mean"][:,2]+self.m_normativeData["FootProgressAngles"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)

        return self.fig

class KineticsPlotComparisonViewer(plotViewers.AbstractPlotViewer):
    """


    """

    def __init__(self,iAnalyses,context,legends,pointLabelSuffix_lst=None,bodyPart= enums.BodyPartPlot.LowerLimb):

        """
            :Parameters:
                 - `iAnalyses` (list ) - list of ` of pyCGM2.Processing.analysis.Analysis`
                 - `context` (str ) - list of ` of pyCGM2.Processing.analysis.Analysis`
                 - `pointLabelSuffix` (str) - suffix ending conventional kinetic CGM labels
    """


        super(KineticsPlotComparisonViewer, self).__init__(iAnalyses)

        for itAnalysis in iAnalyses:
            if isinstance(itAnalysis,pyCGM2.Processing.analysis.Analysis):
                pass
            else:
                logging.error( "[pyCGM2] error input object type. must be a pyCGM2.Core.Processing.analysis.Analysis")

        self.m_analysis = self.m_input
        self.m_context = context
        self.m_pointLabelSuffixes = pointLabelSuffix_lst
        self.m_normativeData = None
        self.m_flagConsistencyOnly = False
        self.m_legends = legends
        self.m_bodyPart=bodyPart

        if len(iAnalyses) != len(legends):
            raise Exception("legends don t match analysis. Must have same length")
        if pointLabelSuffix_lst is not None:
            if len(iAnalyses) != len(pointLabelSuffix_lst):
                raise Exception("list of point label suffix don t match analysis. Must have same length")

        self.m_concretePlotFunction = None

    def setConcretePlotFunction(self, concreteplotFunction):
        self.m_concretePlotFunction = concreteplotFunction

    def __setLayer(self):

        self.fig = plt.figure(figsize=(8.27,11.69), dpi=100,facecolor="white")

        if self.m_concretePlotFunction.func_name in ["descriptivePlot","gaitDescriptivePlot"]:
            title=u""" Descriptive Time-normalized Kinetics \n """
        elif self.m_concretePlotFunction.func_name in ["consistencyPlot","gaitConsistencyPlot"]:
            title=u""" Consistency Time-normalized Kinetics \n """
        else :
            title=u"""\n"""

        self.fig.suptitle(title)
        plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=0.5)

        ax0 = plt.subplot(3,4,1)# Hip X extensor
        ax1 = plt.subplot(3,4,2)# Hip Y abductor
        ax2 = plt.subplot(3,4,3)# Hip Z rotation
        ax3 = plt.subplot(3,4,4)# Knee Z power

        ax4 = plt.subplot(3,4,5)# knee X extensor
        ax5 = plt.subplot(3,4,6)# knee Y abductor
        ax6 = plt.subplot(3,4,7)# knee Z rotation
        ax7 = plt.subplot(3,4,8)# knee Z power

        ax8 = plt.subplot(3,4,9)# ankle X plantar flexion
        ax9 = plt.subplot(3,4,10)# ankle Y rotation
        ax10 = plt.subplot(3,4,11)# ankle Z everter
        ax11 = plt.subplot(3,4,12)# ankle Z power

        if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
            ax0.set_title("Hip extensor Moment" ,size=8)
            ax1.set_title("Hip abductor Moment" ,size=8)
            ax2.set_title("Hip rotation Moment" ,size=8)
            ax3.set_title("Hip Power" ,size=8)

            ax4.set_title("Knee extensor Moment" ,size=8)
            ax5.set_title("Knee abductor Moment" ,size=8)
            ax6.set_title("Knee rotation Moment" ,size=8)
            ax7.set_title("Knee Power" ,size=8)

            ax8.set_title("Ankle plantarflexor Moment" ,size=8)
            ax9.set_title("Ankle everter Moment" ,size=8)
            ax10.set_title("Ankle abductor Moment" ,size=8)
            ax11.set_title("Ankle Power " ,size=8)

            for ax in [self.fig.axes[0],self.fig.axes[1],self.fig.axes[2],
                       self.fig.axes[4],self.fig.axes[5],self.fig.axes[0],
                       self.fig.axes[8],self.fig.axes[9],self.fig.axes[10]]:
                ax.set_ylabel("moment (N.mm.kg-1)",size=8)

            for ax in [self.fig.axes[3],self.fig.axes[7],self.fig.axes[8]]:
                ax.set_ylabel("power (W.Kg-1)",size=8)

            ax8.set_xlabel("Cycle %",size=8)
            ax9.set_xlabel("Cycle %",size=8)
            ax10.set_xlabel("Cycle %",size=8)
            ax11.set_xlabel("Cycle %",size=8)

            if self.m_automaticYlim_flag:
                ax0.set_ylim([-2.0 *1000.0, 3.0*1000.0])
                ax1.set_ylim([-2.0*1000.0, 1.0*1000.0])
                ax2.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax3.set_ylim(  [-3.0, 3.0])

                ax4.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax5.set_ylim([-1.0*1000.0, 1.0*1000.0])
                ax6.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax7.set_ylim( [-3.0, 3.0])

                ax8.set_ylim([-1.0*1000.0, 3.0*1000.0])
                ax9.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax10.set_ylim([-0.5*1000.0, 0.5*1000.0])
                ax11.set_ylim( [-2.0, 5.0])

        else:
            raise Exception ("Plot Panel not implemented yet")

    def __setLegend(self,axisIndex):
        self.fig.axes[axisIndex].legend(fontsize=6)
        #self.fig.axes[axisIndex].legend(fontsize=6, bbox_to_anchor=(0,1.2,1,0.2), loc="lower left",
        #    mode="None", borderaxespad=0, ncol=len(self.m_analysis))

    def setNormativeDataset(self,iNormativeDataSet):
        """
            **Description :** set a normative gait dataset

            :Parameters:
                 - `iNormativeDataSet` (a class of the pyCGM2.Report.normativeDataset module) - normative gait dataset from pyCGM2.Report.normativeDatabaseProcedure module

        """
        # iNormativeDataSet.constructNormativeData()
        self.m_normativeData = iNormativeDataSet.data

    def __setData(self):

        if self.m_context == "Left":
            colormap = plt.cm.Reds
            colormap_i=[colormap(i) for i in np.linspace(0.2, 1, len(self.m_analysis))]

            i = 0
            for analysis in self.m_analysis:

                if self.m_pointLabelSuffixes is not None:
                    suffixPlus = "_" + self.m_pointLabelSuffixes[i] if self.m_pointLabelSuffixes[i] !="" else ""
                else:
                    suffixPlus=""

                legend= self.m_legends[i]

                if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                    # hip
                    self.m_concretePlotFunction(self.fig.axes[0],analysis.kineticStats,
                            "LHipMoment"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None,
                            legendLabel=legend)

                    self.m_concretePlotFunction(self.fig.axes[1],analysis.kineticStats,
                            "LHipMoment"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)


                    self.m_concretePlotFunction(self.fig.axes[2],analysis.kineticStats,
                            "LHipMoment"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[3],analysis.kineticStats,
                            "LHipPower"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    # knee
                    self.m_concretePlotFunction(self.fig.axes[4],analysis.kineticStats,
                            "LKneeMoment"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[5],analysis.kineticStats,
                            "LKneeMoment"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[6],analysis.kineticStats,
                            "LKneeMoment"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[7],analysis.kineticStats,
                            "LKneePower"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    # ankle
                    self.m_concretePlotFunction(self.fig.axes[8],analysis.kineticStats,
                            "LAnkleMoment"+suffixPlus,"Left",0, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[9],analysis.kineticStats,
                            "LAnkleMoment"+suffixPlus,"Left",1, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[10],analysis.kineticStats,
                            "LAnkleMoment"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[11],analysis.kineticStats,
                            "LAnklePower"+suffixPlus,"Left",2, color=colormap_i[i], customLimits=None)
                i+=1

        if self.m_context == "Right":
            colormap = plt.cm.Blues
            colormap_i=[colormap(i) for i in np.linspace(0.2, 1, len(self.m_analysis))]

            i = 0
            for analysis in self.m_analysis:

                if self.m_pointLabelSuffixes is not None:
                    suffixPlus = "_" + self.m_pointLabelSuffixes[i] if self.m_pointLabelSuffixes[i] !="" else ""
                else:
                    suffixPlus=""

                legend= self.m_legends[i]

                if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                    # hip
                    self.m_concretePlotFunction(self.fig.axes[0],analysis.kineticStats,
                            "RHipMoment"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None,
                            legendLabel=legend)

                    self.m_concretePlotFunction(self.fig.axes[1],analysis.kineticStats,
                            "RHipMoment"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)


                    self.m_concretePlotFunction(self.fig.axes[2],analysis.kineticStats,
                            "RHipMoment"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[3],analysis.kineticStats,
                            "RHipPower"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    # knee
                    self.m_concretePlotFunction(self.fig.axes[4],analysis.kineticStats,
                            "RKneeMoment"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[5],analysis.kineticStats,
                            "RKneeMoment"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[6],analysis.kineticStats,
                            "RKneeMoment"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[7],analysis.kineticStats,
                            "RKneePower"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    # ankle
                    self.m_concretePlotFunction(self.fig.axes[8],analysis.kineticStats,
                            "RAnkleMoment"+suffixPlus,"Right",0, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[9],analysis.kineticStats,
                            "RAnkleMoment"+suffixPlus,"Right",1, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[10],analysis.kineticStats,
                            "RAnkleMoment"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                    self.m_concretePlotFunction(self.fig.axes[11],analysis.kineticStats,
                            "RAnklePower"+suffixPlus,"Right",2, color=colormap_i[i], customLimits=None)

                i+=1

        #
    def plotPanel(self):

        if self.m_concretePlotFunction is None:
            raise Exception ("[pyCGM2] need definition of the concrete plot function")

        self.__setLayer()
        self.__setData()
        self.__setLegend(0)

        if self.m_normativeData is not None:
            if self.m_bodyPart == enums.BodyPartPlot.LowerLimb:
                self.fig.axes[0].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipMoment"]["mean"][:,0]-self.m_normativeData["HipMoment"]["sd"][:,0],
                    self.m_normativeData["HipMoment"]["mean"][:,0]+self.m_normativeData["HipMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[1].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipMoment"]["mean"][:,1]-self.m_normativeData["HipMoment"]["sd"][:,1],
                    self.m_normativeData["HipMoment"]["mean"][:,1]+self.m_normativeData["HipMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[3].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["HipPower"]["mean"][:,2]-self.m_normativeData["HipPower"]["sd"][:,2],
                    self.m_normativeData["HipPower"]["mean"][:,2]+self.m_normativeData["HipPower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)


                self.fig.axes[4].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["KneeMoment"]["mean"][:,0]-self.m_normativeData["KneeMoment"]["sd"][:,0],
                    self.m_normativeData["KneeMoment"]["mean"][:,0]+self.m_normativeData["KneeMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[5].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["KneeMoment"]["mean"][:,1]-self.m_normativeData["KneeMoment"]["sd"][:,1],
                    self.m_normativeData["KneeMoment"]["mean"][:,1]+self.m_normativeData["KneeMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[7].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["KneePower"]["mean"][:,2]-self.m_normativeData["KneePower"]["sd"][:,2],
                    self.m_normativeData["KneePower"]["mean"][:,2]+self.m_normativeData["KneePower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)



                self.fig.axes[8].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["AnkleMoment"]["mean"][:,0]-self.m_normativeData["AnkleMoment"]["sd"][:,0],
                    self.m_normativeData["AnkleMoment"]["mean"][:,0]+self.m_normativeData["AnkleMoment"]["sd"][:,0],
                    facecolor="green", alpha=0.5,linewidth=0)

                self.fig.axes[9].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["AnkleMoment"]["mean"][:,1]-self.m_normativeData["AnkleMoment"]["sd"][:,1],
                    self.m_normativeData["AnkleMoment"]["mean"][:,1]+self.m_normativeData["AnkleMoment"]["sd"][:,1],
                    facecolor="green", alpha=0.5,linewidth=0)


                self.fig.axes[11].fill_between(np.linspace(0,100,51),
                    self.m_normativeData["AnklePower"]["mean"][:,2]-self.m_normativeData["AnklePower"]["sd"][:,2],
                    self.m_normativeData["AnklePower"]["mean"][:,2]+self.m_normativeData["AnklePower"]["sd"][:,2],
                    facecolor="green", alpha=0.5,linewidth=0)

        return self.fig



# -*- coding: utf-8 -*-
import logging
import sys
from logging import handlers
#logging.basicConfig(filename = "installer.log", level=logging.DEBUG)

def setLoggingLevel(level):
    logging.basicConfig(format = "[pyCGM2-%(levelname)s]-%(module)s-%(funcName)s : %(message)s",level = level)

def setLogger(filename = "pyCGM2.log",level = logging.INFO,mode="erase" ):
    log = logging.getLogger('')
    log.setLevel(level)
    format = logging.Formatter("[pyCGM2-%(levelname)s]-%(module)s-%(funcName)s : %(message)s")
    ch = logging.StreamHandler(sys.stdout)
    ch.setFormatter(format)
    log.addHandler(ch)
    if mode =="erase":
        fh = logging.FileHandler(filename, mode='w')
    else:
        fh = handlers.RotatingFileHandler(filename, maxBytes=(1048576*5), backupCount=7)
    fh.setFormatter(format)
    log.addHandler(fh)



# -*- coding: utf-8 -*-
from pyCGM2.Configurator import EmgManager
from pyCGM2.Configurator import ModelManager

# -----emg------
def getEmgConfiguration(userSettings,internalSettings):
    """
    return emg settings

    :param userSettings [dict]:  content of the userSettings yaml file
    :param internalSettings [dict]: content of the internalSettings yaml file

    **Return**
    :param [dict]:  eventual emg settings

    """
    # --- Manager ----
    manager = EmgManager.EmgConfigManager(userSettings,localInternalSettings=internalSettings)
    manager.contruct()

    return manager.getEmgConfiguration()


# -----Model------

def getCGMmanager(CGMversion,userSettings,internalSettings=None,translators=None,localIkWeight=None,vsk=None):
    """
    return CGM settings

    :param CGMversion [str]:  CGM version name
    :param userSettings [dict]:  content of the userSettings yaml file


    **optional**
    :param internalSettings [dict]: content of the internalSettings yaml file
    :param translators [dict]: content of the translators file
    :param localIkWeight [dict]: content of the localIkweight file
    :param vsk [file]: vsk file

    **Return**
    :param [dict]:  eventual CGM settings

    """

    # --- Manager ----
    if model  == "CGM1.0":
        manager = ModelManager.CGM1ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)

    elif model  == "CGM1.1":
        manager = ModelManager.CGM1_1ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)

    elif model  == "CGM2.1":
        manager = ModelManager.CGM2_1ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)

    elif model  == "CGM2.2":
        manager = ModelManager.CGM2_2ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)

    elif model  == "CGM2.3":
        manager = ModelManager.CGM2_3ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)

    elif model  == "CGM2.4":
        manager = ModelManager.CGM2_4ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)


    elif model  == "CGM2.5":
        manager = ModelManager.CGM2_5ConfigManager(userSettings,
                                                    localInternalSettings=internalSettings,
                                                    localTranslators=translators,
                                                    localIkWeight=localIkWeight,
                                                    vsk=vsk)
    else:
        raise Exception ("[pyCGM2] : Model version not known (choice CGM1.0 to CGM2.5 )")

    manager.contruct()

    return manager



# -*- coding: utf-8 -*-
import pyCGM2
import logging

def toBool(text):
    return True if text == "True" else False

def isInRange(val, min, max):

    if val<min or val>max:
        return False
    else:
        return True

def str(unicodeVariable):
    return unicodeVariable.encode(pyCGM2.ENCODER)

def checkSimilarElement(listData):
    if(len(set(listData))==1):
        for it in set(listData):
            return it
        return True
    else:
        logging.error("[pyCGM2] items are different in the inputed list" )
        return False

def getSimilarElement(listData):
    out = list()
    for it in set(listData):
        out = it
    return out



# -*- coding: utf-8 -*-

import pyCGM2

import numpy as np
from pyCGM2.Tools import btkTools
from pyCGM2.Nexus import nexusTools
import matplotlib.pyplot as plt




# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import logging
import numpy as np

from pyCGM2 import btk

from pyCGM2.Tools import  btkTools
from pyCGM2.Signal import detect_peaks


#-------- EVENT PROCEDURES  ----------


# --- calibration procedure
class LowDimensionalKalmanFilterProcedure(object):
    """
        method from Burke et al. (Job 2016)
    """

    def __init__(self):
        self.description = "Burke (2016)"


    def _smooth(self,rawdata,tol=0.0025,sigR=1e-3,keepOriginal=True):

    	X = rawdata[~np.isnan(rawdata).any(axis=1)]

    	m = np.mean(X,axis=0)

    	logging.debug('Computing SVD...')

    	U, S, V = np.linalg.svd(X - m)

    	logging.debug('done')

    	d = np.nonzero(np.cumsum(S)/np.sum(S)>(1-tol))[0][0]

    	Q = np.dot(np.dot(V[0:d,:],np.diag(np.std(np.diff(X,axis=0),axis=0))),V[0:d,:].T)

    	logging.debug('Forward Pass')
    	state = []
    	state_pred = []
    	cov_pred = []
    	cov = []
    	cov.insert(0,1e12*np.eye(d))
    	state.insert(0,np.random.normal(0.0,1.0,d))
    	cov_pred.insert(0,1e12*np.eye(d))
    	state_pred.insert(0,np.random.normal(0.0,1.0,d))
    	for i in range(1,rawdata.shape[0]+1):

    		z =  rawdata[i-1,~np.isnan(rawdata[i-1,:])]
    		H = np.diag(~np.isnan(rawdata[i-1,:]))
    		H = H[~np.all(H == 0, axis=1)]
    		Ht = np.dot(H,V[0:d,:].T)

    		R = sigR*np.eye(H.shape[0])

    		state_pred.insert(i,state[i-1])
    		cov_pred.insert(i,cov[i-1] + Q)

    		K = np.dot(np.dot(cov_pred[i],Ht.T),np.linalg.inv(np.dot(np.dot(Ht,cov_pred[i]),Ht.T) + R))

    		state.insert(i,state_pred[i] + np.dot(K,(z - (np.dot(Ht,state_pred[i])+np.dot(H,m)))))
    		cov.insert(i,np.dot(np.eye(d) - np.dot(K,Ht),cov_pred[i]))

    	logging.debug('Backward Pass')
    	y = np.zeros(rawdata.shape)
    	y[-1,:] = np.dot(V[0:d,:].T,state[-1]) + m
    	for i in range(len(state)-2,0,-1):
    		state[i] =  state[i] + np.dot(np.dot(cov[i],np.linalg.inv(cov_pred[i])),(state[i+1] - state_pred[i+1]))
    		cov[i] =  cov[i] + np.dot(np.dot(np.dot(cov[i],np.linalg.inv(cov_pred[i])),(cov[i+1] - cov_pred[i+1])),cov[i])

    		y[i-1,:] = np.dot(V[0:d,:].T,state[i]) + m

    	if (keepOriginal):
    		y[~np.isnan(rawdata)] = rawdata[~np.isnan(rawdata)]

    	return y


    def fill(self,acq):
        logging.info("----LowDimensionalKalmanFilter gap filling----")
        btkmarkersLoaded  = btkTools.GetMarkerNames(acq)
        ff = acq.GetFirstFrame()
        lf = acq.GetLastFrame()
        pfn = acq.GetPointFrameNumber()

        btkmarkers =[]
        for ml in btkmarkersLoaded:
            if btkTools.isPointExist(acq,ml) :
                btkmarkers.append(ml)

        # --------

        logging.debug("Populating data matrix")
        rawDatabtk = np.zeros((pfn,len(btkmarkers)*3))
        for i in range(0,len(btkmarkers)):
            values = acq.GetPoint(btkmarkers[i]).GetValues()
            residualValues = acq.GetPoint(btkmarkers[i]).GetResiduals()
            rawDatabtk[:,3*i-3] = values[:,0]
            rawDatabtk[:,3*i-2] = values[:,1]
            rawDatabtk[:,3*i-1] = values[:,2]
            E = residualValues[:,0]
            rawDatabtk[np.asarray(E)==-1,3*i-3] = np.nan
            rawDatabtk[np.asarray(E)==-1,3*i-2] = np.nan
            rawDatabtk[np.asarray(E)==-1,3*i-1] = np.nan

        Y2 = self._smooth(rawDatabtk,tol =1e-2,sigR=1e-3,keepOriginal=True)

        logging.debug("writing trajectories")

        # Create new smoothed trjectories
        filledMarkers  = list()
        for i in range(0,len(btkmarkers)):
            targetMarker = btkmarkers[i]
            if btkTools.isGap(acq,targetMarker):
                logging.info("marker (%s) --> filled"%(targetMarker))
                filledMarkers.append(targetMarker)
                val_final = np.zeros((pfn,3))
                val_final[:,0] = Y2[:,3*i-3]
                val_final[:,1] = Y2[:,3*i-2]
                val_final[:,2] = Y2[:,3*i-1]
                btkTools.smartAppendPoint(acq,targetMarker,val_final)
        logging.info("----LowDimensionalKalmanFilter gap filling [complete]----")

        return acq, filledMarkers


class GapFillingFilter(object):
    """

    """
    def __init__(self,procedure,acq):
        """
            :Parameters:
        """

        self.m_aqui = acq
        self.m_procedure = procedure

        self.filledMarkers  = None
        self.filledAcq  = None

    def getFilledAcq(self):
        return self.filledAcq

    def getFilledMarkers(self):
        return self.filledMarkers


    def fill(self):
        """

        """
        filledAcq,filledMarkers = self.m_procedure.fill(self.m_aqui)


        self.filledMarkers  = filledMarkers
        self.filledAcq  = filledAcq



# -*- coding: utf-8 -*-
import logging
import numpy as np

from pyCGM2 import btk
from pyCGM2 import opensim3 as opensim
from pyCGM2.Utils import utils




R_OSIM_CGM = {"Pelvis" : np.array([[1,0,0],[0,0,1],[0,-1,0]]) ,
              "Left Thigh" : np.array([[1,0,0],[0,0,1],[0,-1,0]]),
              "Left Shank" : np.array([[1,0,0],[0,0,1],[0,-1,0]]),
              "Right Thigh" : np.array([[1,0,0],[0,0,1],[0,-1,0]]),
              "Right Shank" : np.array([[1,0,0],[0,0,1],[0,-1,0]]),
              "Left Foot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Right Foot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Right HindFoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Right ForeFoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Left HindFoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Left ForeFoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]),
              "Right Hindfoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]]), # use for julie's model
              "Right Forefoot" : np.array([[0,0,-1],[1,0,0],[0,-1,0]])

              }




def globalTransformationLabToOsim(acq,R_LAB_OSIM):

    points = acq.GetPoints()
    for it in btk.Iterate(points):
        if it.GetType() == btk.btkPoint.Marker:
            values = np.zeros(it.GetValues().shape)
            for i in range(0,it.GetValues().shape[0]):
                values[i,:] = np.dot(R_LAB_OSIM,it.GetValues()[i,:])
            it.SetValues(values)

def smartTrcExport(acq,filenameNoExt):
    writerDyn = btk.btkAcquisitionFileWriter()
    writerDyn.SetInput(acq)
    writerDyn.SetFilename(utils.str(filenameNoExt + ".trc"))
    writerDyn.Update()



def sto2pointValues(stoFilename,label,R_LAB_OSIM):
    storageObject = opensim.Storage(utils.str(stoFilename))
    index_x =storageObject.getStateIndex(label+"_tx")
    index_y =storageObject.getStateIndex(label+"_ty")
    index_z =storageObject.getStateIndex(label+"_tz")

    array_x=opensim.ArrayDouble()
    storageObject.getDataColumn(index_x,array_x)

    array_y=opensim.ArrayDouble()
    storageObject.getDataColumn(index_y,array_y)

    array_z=opensim.ArrayDouble()
    storageObject.getDataColumn(index_z,array_z)

    n= array_x.getSize()
    pointValues = np.zeros((n,3))
    for i in range(0,n):
        pointValues[i,0] = array_x.getitem(i)
        pointValues[i,1] = array_y.getitem(i)
        pointValues[i,2] = array_z.getitem(i)


    for i in range(0,n):
        pointValues[i,:] = np.dot(R_LAB_OSIM.T,pointValues[i,:])*1000.0


    return pointValues


def mot2pointValues(motFilename,labels,orientation =[1,1,1]):
    storageObject = opensim.Storage(motFilename)

    index_x =storageObject.getStateIndex(labels[0])
    index_y =storageObject.getStateIndex(labels[1])
    index_z =storageObject.getStateIndex(labels[2])

    array_x=opensim.ArrayDouble()
    storageObject.getDataColumn(index_x,array_x)

    array_y=opensim.ArrayDouble()
    storageObject.getDataColumn(index_y,array_y)

    array_z=opensim.ArrayDouble()
    storageObject.getDataColumn(index_z,array_z)

    n= array_x.getSize()
    pointValues = np.zeros((n,3))
    for i in range(0,n):
        pointValues[i,0] = orientation[0]*array_x.getitem(i)
        pointValues[i,1] = orientation[1]*array_y.getitem(i)
        pointValues[i,2] = orientation[2]*array_z.getitem(i)


    return pointValues

def setGlobalTransormation_lab_osim(axis,forwardProgression):
    """ Todo : incomplet, il faut traiter tous les cas """
    if axis =="X":
        if forwardProgression:
            R_LAB_OSIM=np.array([[1,0,0],[0,0,1],[0,-1,0]])
        else:
            R_LAB_OSIM=np.array([[-1,0,0],[0,0,1],[0,1,0]])

    elif axis =="Y":
        if forwardProgression:
            R_LAB_OSIM=np.array([[0,1,0],[0,0,1],[1,0,0]])
        else:
            R_LAB_OSIM=np.array([[0,-1,0],[0,0,1],[-1,0,0]])

    else:
        raise Exception("[pyCGM2] - Global Referential not configured yet")


    return R_LAB_OSIM


class opensimModel(object):


    def __init__(self,osimFile,cgmModel,):
        self.m_osimFile = osimFile


        self.m_model = opensim.Model(utils.str(osimFile))
        self.m_cgmModel = cgmModel
        self.m_markers= list()


        self.m_model.initSystem()
        self.m_myState = self.m_model.initSystem()




    def addMarkerSet(self,markersetFile):
        markerSet= opensim.MarkerSet(markersetFile)
        self.m_model.replaceMarkerSet(self.m_myState,markerSet)

    def setOsimJoinCentres(self,R_OSIM_CGM, jointLabelInOsim, parentSegmentLabel,childSegmentLabel, nodeLabel, toMeter = 1000.0):

        locationInParent =  self.m_cgmModel.getSegment(parentSegmentLabel).anatomicalFrame.static.getNode_byLabel(nodeLabel).m_local # TODO : verifier que node exist
        locationInChild =  self.m_cgmModel.getSegment(childSegmentLabel).anatomicalFrame.static.getNode_byLabel(nodeLabel).m_local # TODO : verifier que node exist

        positionParent =   np.dot(R_OSIM_CGM[parentSegmentLabel],locationInParent)
        positionChild =   np.dot(R_OSIM_CGM[childSegmentLabel],locationInChild)

        osimJointInParent = self.m_model.getJointSet().get(jointLabelInOsim).get_location_in_parent()
        osimJointInParent.set(0,positionParent[0]/toMeter)
        osimJointInParent.set(1,positionParent[1]/toMeter)
        osimJointInParent.set(2,positionParent[2]/toMeter)

        osimJointInChild = self.m_model.getJointSet().get(jointLabelInOsim).get_location()
        osimJointInChild.set(0,positionChild[0]/toMeter)
        osimJointInChild.set(1,positionChild[1]/toMeter)
        osimJointInChild.set(2,positionChild[2]/toMeter)

        logging.debug( "osim joint centres %s modified"%(jointLabelInOsim))



    def addmarkerFromModel(self, label, osimBodyName = "", modelSegmentLabel = "", rotation_osim_model=np.eye(3), toMeter=1000.0):

        if (osimBodyName != ""  and  modelSegmentLabel != "") :
            localPos = np.dot(rotation_osim_model,
                               self.m_cgmModel.getSegment(modelSegmentLabel).anatomicalFrame.static.getNode_byLabel(label).m_local) # TODO : verifier que node exist

            m = opensim.Marker()
            m.setOffset(opensim.Vec3(localPos[0]/toMeter,localPos[1]/toMeter,localPos[2]/toMeter))
            m.setName(label)
            m.setBodyName(osimBodyName)
            m.setFixed(False)

            self.m_markers.append(m)

        logging.debug( "marker (%s) added"%(label))


    def createMarkerSet(self):

        for i in range(0,len(self.m_markers)):
            self.m_model.getMarkerSet().set(i,self.m_markers[i]) # FIXME  : this line cause a memory issue : it would be better to use add_maker but i have an error with const double[3] when i specified the offset


    def updateMarkerInMarkerSet(self, label, modelSegmentLabel = "", rotation_osim_model=np.eye(3), toMeter=1000.0):

       #if label =="LTOE": pdb.set_trace()
       markers = opensim.ArrayStr()
       self.m_model.getMarkerSet().getMarkerNames(markers)

       index = markers.findIndex(label)
       if index != -1:
           logging.debug( "marker (%s) found"%(label))
           localPos = np.dot(rotation_osim_model,
                                  self.m_cgmModel.getSegment(modelSegmentLabel).anatomicalFrame.static.getNode_byLabel(label).m_local) # TODO : check node exist

           self.m_model.updMarkerSet().get(label).getOffset().set(0,localPos[0]/toMeter)
           self.m_model.updMarkerSet().get(label).getOffset().set(1,localPos[1]/toMeter)
           self.m_model.updMarkerSet().get(label).getOffset().set(2,localPos[2]/toMeter)

       else:
           raise Exception ("[pyCGM2] marker (%s) is not within the markerset"%label)





class opensimKinematicFitting(object):

    def __init__(self,osimModel,ikToolFiles):
        self.m_model = osimModel

        ikTool =  opensim.InverseKinematicsTool(ikToolFiles)
        self.m_ikMarkers = list()

        self.m_ikTool = ikTool#opensim.InverseKinematicsTool()
        pr= self.m_ikTool.getPropertyByName("report_marker_locations")
        opensim.PropertyHelper().setValueBool(True,pr)

        #self.setAccuracy(1e-08)

    def setAccuracy(self,value):
        pr= self.m_ikTool.getPropertyByName("accuracy")
        opensim.PropertyHelper().setValueDouble(value,pr)

    def setResultsDirectory(self,path):
        pr= self.m_ikTool.getPropertyByName("results_directory")
        opensim.PropertyHelper().setValueString(utils.str(path),pr)

    def addIkMarkerTask(self,label,weight=100):
        markerTask = opensim.IKMarkerTask()
        markerTask.setName(label)
        markerTask.setWeight(weight)
        self.m_ikMarkers.append(markerTask)

    def createIKTaskSet(self):
        for i in range(0,len(self.m_ikMarkers)):
            self.m_ikTool.getIKTaskSet().set(int(i),self.m_ikMarkers[i]) #FIXME memory issue


    def config(self, R_LAB_OSIM,acq, acqFileName):

        # steps 1 : motion data processing
        globalTransformationLabToOsim(acq,R_LAB_OSIM) # global Transformation lab to OSIM

        filenameNoExt = acqFileName[:-4]
        smartTrcExport(acq,filenameNoExt) # trc export

        # steps 2 : config ikTool
        self.m_ikTool.setModel(self.m_model)
        self.m_ikTool.setMarkerDataFileName(utils.str(filenameNoExt+".trc"))
        self.m_ikTool.setOutputMotionFileName(utils.str(filenameNoExt+".mot"))

        #  set times ( FIXME - I had surprise with set method, i prefer to handle the xmlnode directly)
        prTime= self.m_ikTool.getPropertyByName("time_range")

        opensim.PropertyHelper().appendValueDouble(0.0, prTime)
        endTime = (acq.GetLastFrame() - acq.GetFirstFrame())/acq.GetPointFrequency()
        opensim.PropertyHelper().appendValueDouble(endTime, prTime)

        # doesn t work
        # markerData = opensim.MarkerData(filenameNoExt+".trc")
        # self.m_ikTool.setStartTime(markerData.getStartFrameTime()) # time not frame !
        # self.m_ikTool.setEndTime(markerData.getLastFrameTime())


    def updateIKTask(self,label,weight):
        ts =self.m_ikTool.getIKTaskSet()
        index = ts.getIndex(label)
        if index !=-1 :
            if weight != 0:
                ts.get(label).setApply(True)
                ts.get(label).setWeight(weight)
            else:
                ts.get(label).setApply(False)
        else:
            raise Exception("[[pyCGM2]] the label (%s) doesn t exist "%(label))

    def run(self):

        self.m_ikTool.run()



# -*- coding: utf-8 -*-
import pyCGM2
from pyCGM2.Configurator import Manager
from pyCGM2.Utils import files
from pyCGM2 import enums
import logging
import copy
from pyCGM2.Report import normativeDatasets


class GaitProcessingConfigManager(Manager.ConfigManager):
    """

    """
    def __init__(self,settings, modelVersion = None):
        super(GaitProcessingConfigManager, self).__init__(settings)
        self.modelVersion = modelVersion

    @property
    def task(self):
        return self._userSettings["Task"]

    @property
    def conditions(self):
        return self._userSettings["Conditions"]


    @property
    def experimentalInfo(self):
        return self._userSettings["ExperimentalInfo"]

    @property
    def subjectInfo(self):
        return self._userSettings["SubjectInfo"]

    @property
    def modelledTrials(self):
        return self._userSettings["ModelledTrials"]


    @property
    def normativeData(self):

        normativeData = {"Author" : self._userSettings["NormativeData"]["Author"],
                        "Modality" : self._userSettings["NormativeData"]["Modality"]}

        if normativeData["Author"] == "Schwartz2008":
            chosenModality = normativeData["Modality"]
            nds = normativeDatasets.Schwartz2008(chosenModality)    # modalites : "Very Slow" ,"Slow", "Free", "Fast", "Very Fast"
        elif normativeData["Author"] == "Pinzone2014":
            chosenModality = normativeData["Modality"]
            nds = normativeDatasets.Pinzone2014(chosenModality)

        return nds

    @property
    def pointSuffix(self):
        value = self._userSettings["Point suffix"]
        return  None if value=="None" else value

    @property
    def consistencyFlag(self):
        return self._userSettings["Consistency"]

    @property
    def title(self):
        if self._userSettings["Title"] == "None":

            if self.modelVersion is not None:
                title = self.modelVersion +"-"+self._userSettings["Task"]+ "-Gait Processing"
            else:
                title = self._userSettings["Task"]+ "-Gait Processing"

            self._userSettings["Title"] = title
            return self._userSettings["Title"]
        else:
            return self._userSettings["Title"]

    @property
    def bodyPart(self):
        return self._userSettings["BodyPart"]



# -*- coding: utf-8 -*-
import numpy as np
import logging
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches


# pyCGM2
#import pyCGM2
from pyCGM2.Processing import cycle

from pyCGM2.Tools import btkTools
from pyCGM2.Report import plotUtils


from pyCGM2 import btk

from pyCGM2.EMG import normalActivation
from pyCGM2.Utils import utils

# ---- convenient plot functions
def temporalPlot(figAxis,acq,
                pointLabel,axis,pointLabelSuffix=None,color=None,
                title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `acq` (ma.Trial) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

    '''

    pointLabel = pointLabel + "_" + pointLabelSuffix if pointLabelSuffix is not None else pointLabel


    flag = btkTools.isPointExist(acq,pointLabel)
    if flag:
        point = acq.GetPoint(utils.str(pointLabel))
        lines=figAxis.plot(point.GetValues()[:,axis], '-', color= color)
        appf = 1
    else:
        flag = btkTools.isAnalogExist(acq,pointLabel)
        if flag:
            analog = acq.GetAnalog(utils.str(pointLabel))
            lines=figAxis.plot(analog.GetValues()[:,axis], '-', color= color)
            appf = acq.GetNumberAnalogSamplePerFrame()


    if legendLabel is not None  and flag: lines[0].set_label(legendLabel)
    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None: figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None: figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None: figAxis.set_ylim(ylim)

    if flag:
        for ev in btk.Iterate( acq.GetEvents()):
            colorContext = plotUtils.colorContext(ev.GetContext())
            if ev.GetLabel() == "Foot Strike":
                figAxis.axvline( x= (ev.GetFrame()-acq.GetFirstFrame())*appf, color = colorContext, linestyle = "-")
            if ev.GetLabel() == "Foot Off":
                figAxis.axvline( x= (ev.GetFrame()-acq.GetFirstFrame())*appf, color = colorContext, linestyle = "--")


def descriptivePlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

   '''


    # check if [ pointlabel , context ] in keys of analysisStructureItem
    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"] != [] else False


    # plot
    if flag:
        mean=analysisStructureItem.data[pointLabel,contextPointLabel]["mean"][:,axis]
        std=analysisStructureItem.data[pointLabel,contextPointLabel]["std"][:,axis]
        line= figAxis.plot(np.linspace(0,100,101), mean, color=color,linestyle="-")
        figAxis.fill_between(np.linspace(0,100,101), mean-std, mean+std, facecolor=color, alpha=0.5,linewidth=0)

        if customLimits is not None:
            for value in customLimits:
                figAxis.axhline(value,color=color,ls='dashed')


    if legendLabel is not None: line[0].set_label(legendLabel)
    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None:figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None:figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None:figAxis.set_ylim(ylim)

def consistencyPlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

    '''

    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            n = len(analysisStructureItem.data[pointLabel,contextPointLabel]["values"])
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"] != [] else False

    # plot
    if flag:
        values= np.zeros((101,n))
        i=0
        for val in analysisStructureItem.data[pointLabel,contextPointLabel]["values"]:
           values[:,i] = val[:,axis]
           i+=1

        lines = figAxis.plot(np.linspace(0,100,101), values, color=color)

        if customLimits is not None:
           for value in customLimits:
               figAxis.axhline(value,color=color,ls='dashed')


    if legendLabel is not None and flag: lines[0].set_label(legendLabel)

    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None:figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None:figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None:figAxis.set_ylim(ylim)


def meanPlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

   '''


    # check if [ pointlabel , context ] in keys of analysisStructureItem
    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"] != [] else False


    # plot
    if flag:
        mean=analysisStructureItem.data[pointLabel,contextPointLabel]["mean"][:,axis]
        lines= figAxis.plot(np.linspace(0,100,101), mean, color=color,linestyle="-")

        if customLimits is not None:
            for value in customLimits:
                figAxis.axhline(value,color=color,ls='dashed')

    if legendLabel is not None  and flag: lines[0].set_label(legendLabel)
    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None: figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None: figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None: figAxis.set_ylim(ylim)


def gaitDescriptivePlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

   '''


    # check if [ pointlabel , context ] in keys of analysisStructureItem
    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"]!=[]  else False

    # plot
    if flag:
        mean=analysisStructureItem.data[pointLabel,contextPointLabel]["mean"][:,axis]
        std=analysisStructureItem.data[pointLabel,contextPointLabel]["std"][:,axis]
        line= figAxis.plot(np.linspace(0,100,101), mean, color=color,linestyle="-")
        figAxis.fill_between(np.linspace(0,100,101), mean-std, mean+std, facecolor=color, alpha=0.5,linewidth=0)

        # add gait phases
        stance = analysisStructureItem.pst['stancePhase', contextPointLabel]["mean"]
        double1 = analysisStructureItem.pst['doubleStance1', contextPointLabel]["mean"]
        double2 = analysisStructureItem.pst['doubleStance2', contextPointLabel]["mean"]
        figAxis.axvline(stance,color=color,ls='dashed')
        figAxis.axvline(double1,ymin=0.9, ymax=1.0,color=color,ls='dotted')
        figAxis.axvline(stance-double2,ymin=0.9, ymax=1.0,color=color,ls='dotted')

        if customLimits is not None:
            for value in customLimits:
                figAxis.axhline(value,color=color,ls='dashed')

    if legendLabel is not None: line[0].set_label(legendLabel)
    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None:figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None:figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None:figAxis.set_ylim(ylim)

def gaitConsistencyPlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

    '''

    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            n = len(analysisStructureItem.data[pointLabel,contextPointLabel]["values"])
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"]!=[] else False

    # plot
    if flag:
        values= np.zeros((101,n))
        i=0
        for val in analysisStructureItem.data[pointLabel,contextPointLabel]["values"]:

            values[:,i] = val[:,axis]

            i+=1

        lines = figAxis.plot(np.linspace(0,100,101), values, color=color)

        for valStance,valDouble1,valDouble2, in zip(analysisStructureItem.pst['stancePhase', contextPointLabel]["values"],
                                                    analysisStructureItem.pst['doubleStance1', contextPointLabel]["values"],
                                                    analysisStructureItem.pst['doubleStance2', contextPointLabel]["values"]):

            figAxis.axvline(valStance,color=color,ls='dashed')
            figAxis.axvline(valDouble1,ymin=0.9, ymax =1.0 ,color=color,ls='dotted')
            figAxis.axvline(valStance-valDouble2,ymin=0.9, ymax =1.0 ,color=color,ls='dotted')

        if customLimits is not None:
           for value in customLimits:
               figAxis.axhline(value,color=color,ls='dashed')


    if legendLabel is not None and flag: lines[0].set_label(legendLabel)

    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None:figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None:figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None:figAxis.set_ylim(ylim)

def gaitMeanPlot(figAxis,analysisStructureItem,
                        pointLabel,contextPointLabel,axis,
                        color=None,
                        title=None, xlabel=None, ylabel=None,ylim=None,legendLabel=None,
                        customLimits=None):

    '''

        **Description :** plot descriptive statistical (average and sd corridor) gait traces from a pyCGM2.Processing.analysis.Analysis instance

        :Parameters:
             - `figAxis` (matplotlib::Axis )
             - `analysisStructureItem` (pyCGM2.Processing.analysis.Analysis.Structure) - a Structure item of an Analysis instance built from AnalysisFilter

        :Return:
            - matplotlib figure

        **Usage**

        .. code:: python

   '''


    # check if [ pointlabel , context ] in keys of analysisStructureItem
    flag = False
    for key in analysisStructureItem.data.keys():
        if key[0] == pointLabel and key[1] == contextPointLabel:
            flag = True if analysisStructureItem.data[pointLabel,contextPointLabel]["values"] != [] else False


    # plot
    if flag:
        mean=analysisStructureItem.data[pointLabel,contextPointLabel]["mean"][:,axis]
        lines= figAxis.plot(np.linspace(0,100,101), mean, color=color,linestyle="-")

        stance = analysisStructureItem.pst['stancePhase', contextPointLabel]["mean"]
        double1 = analysisStructureItem.pst['doubleStance1', contextPointLabel]["mean"]
        double2 = analysisStructureItem.pst['doubleStance2', contextPointLabel]["mean"]
        figAxis.axvline(stance,color=color,ls='dashed')
        figAxis.axvline(double1,ymin=0.9, ymax=1.0,color=color,ls='dotted')
        figAxis.axvline(stance-double2,ymin=0.9, ymax=1.0,color=color,ls='dotted')

        if customLimits is not None:
            for value in customLimits:
                figAxis.axhline(value,color=color,ls='dashed')

    if legendLabel is not None  and flag: lines[0].set_label(legendLabel)
    if title is not None: figAxis.set_title(title ,size=8)
    figAxis.set_xlim([0.0,100])
    figAxis.tick_params(axis='x', which='major', labelsize=6)
    figAxis.tick_params(axis='y', which='major', labelsize=6)
    if xlabel is not None: figAxis.set_xlabel(xlabel,size=8)
    if ylabel is not None: figAxis.set_ylabel(ylabel,size=8)
    if ylim is not None: figAxis.set_ylim(ylim)

def stpHorizontalHistogram(figAxis,analysisStructureItem,
                        stpLabel,
                        overall= False,
                        title=None, xlabel=None,xlim=None):

    if (stpLabel,"Right") in analysisStructureItem.keys() and (stpLabel,"Left") in analysisStructureItem.keys():
        overallData = np.concatenate([analysisStructureItem[stpLabel,"Left"]["values"],analysisStructureItem[stpLabel,"Right"]["values"]] )
        mean_L = analysisStructureItem[stpLabel,"Left"]["mean"]
        err_L = analysisStructureItem[stpLabel,"Left"]["std"]
        mean_R = analysisStructureItem[stpLabel,"Right"]["mean"]
        err_R = analysisStructureItem[stpLabel,"Right"]["std"]

    if not (stpLabel,"Right") in analysisStructureItem.keys() and (stpLabel,"Left") in analysisStructureItem.keys():
        overallData = analysisStructureItem[stpLabel,"Left"]["values"]
        mean_L = analysisStructureItem[stpLabel,"Left"]["mean"]
        err_L = analysisStructureItem[stpLabel,"Left"]["std"]
        mean_R = 0
        err_R = 0

    if  (stpLabel,"Right") in analysisStructureItem.keys() and not (stpLabel,"Left") in analysisStructureItem.keys():
        overallData = analysisStructureItem[stpLabel,"Right"]["values"]
        mean_L = 0
        err_L = 0
        mean_R = analysisStructureItem[stpLabel,"Right"]["mean"]
        err_R = analysisStructureItem[stpLabel,"Right"]["std"]

    if overall:
        mean = np.mean(overallData)
        err = np.std(overallData)
        figAxis.barh([0], [ mean], color='purple', xerr=[err])
        figAxis.set_ylabel("Overall",size=8)
        figAxis.set_yticklabels( [""])
        figAxis.text(0,0,round(mean,2))

    else:
        figAxis.barh([0,1], [mean_L,mean_R],  xerr=[err_L,err_R], color=["red","blue"])
        figAxis.set_yticks([0,1])
        figAxis.set_yticklabels(["L","R"],size=8)
        figAxis.text(0,0,round(mean_L,2))
        figAxis.text(0,1,round(mean_R,2))

    if title is not None: figAxis.set_title(title ,size=8)
    if xlabel is not None: figAxis.set_xlabel(xlabel,size=8)
    if xlim is not None: figAxis.set_xlim(xlim)

    figAxis.tick_params(axis='x', which='major', labelsize=6)


def addNormalActivationLayer(figAxis,normalActivationLabel,fo):
    pos,burstDuration=normalActivation.getNormalBurstActivity(normalActivationLabel,fo)
    for j in range(0,len(pos)):
        figAxis.add_patch(plt.Rectangle((pos[j],0),burstDuration[j],figAxis.get_ylim()[1] , color='g',alpha=0.1))



def addTemporalNormalActivationLayer(figAxis,acq,normalActivationLabel,context):
    if normalActivationLabel:
        gaitCycles = cycle.construcGaitCycle(acq)

        for cycleIt  in gaitCycles:
            if cycleIt.context == context:
                pos,burstDuration=normalActivation.getNormalBurstActivity_fromCycles(normalActivationLabel,cycleIt.firstFrame,cycleIt.begin, cycleIt.m_contraFO, cycleIt.end, cycleIt.appf)
                for j in range(0,len(pos)):
                    figAxis.add_patch(plt.Rectangle((pos[j],0),burstDuration[j],figAxis.get_ylim()[1] , color='g',alpha=0.1))



def addRectanglePatches(figAxis,clusters,heightProportion = 0.05):
    ymax = figAxis.get_ylim()[1]
    ymin = figAxis.get_ylim()[0]
    height = (ymax-ymin)*heightProportion
    for cluster in clusters:
        begin = cluster[0]
        end = cluster[1]
        rectanglePatch = mpatches.Rectangle((begin, ymax-height),
                                        end-begin,
                                        height,
                                        color="k")

        figAxis.add_patch(rectanglePatch)



# -*- coding: utf-8 -*-

import pyCGM2
from pyCGM2.Utils import files

def getNormalBurstActivity(muscle, fo):
        """
        detection des zones d"activité normal emg
         voici les on pour l"emg
        #            on1=beg
        #            on2=beg+(12/NORMAL_STANCE_PHASE)*(fo-beg)
        #            on3=beg+(48/NORMAL_STANCE_PHASE)*(fo-beg)
        #            on4=fo+((70-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)
        #            on5=fo+((93-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)
        #            on6=fo+((100-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

        # valeurs normales issues d"une livre EMG analys
        """

        normalActivations = files.openJson(pyCGM2.NORMATIVE_DATABASE_PATH+"emg\\","normalActivation.json")

        NORMAL_STANCE_PHASE=normalActivations["NORMAL_STANCE_PHASE"]
        TABLE = normalActivations["Activation"]

        beg=  0
        end=100

        if muscle in TABLE.keys():
            list_beginBurst=[]
            list_burstDuration=[]

            for i,j in zip(range(0,6,2),range(1,6,2)):
                if TABLE[muscle][i]!="na" and  TABLE[muscle][j]!="na":
                    if TABLE[muscle][i]<NORMAL_STANCE_PHASE:
                        beginBurst=beg+(TABLE[muscle][i]/NORMAL_STANCE_PHASE)*(fo-beg)
                    else:
                        beginBurst=fo+((TABLE[muscle][i]-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

                    if TABLE[muscle][j]<NORMAL_STANCE_PHASE:
                        endBurst=beg+(TABLE[muscle][j]/NORMAL_STANCE_PHASE)*(fo-beg)
                    else:
                        endBurst=fo+((TABLE[muscle][j]-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

                    list_beginBurst.append(beginBurst)
                    list_burstDuration.append(endBurst-beginBurst)

        else:
            list_beginBurst=[0]
            list_burstDuration=[0]

        return list_beginBurst,list_burstDuration


def getNormalBurstActivity_fromCycles(muscle,ff,begin, fo, end, apf):
        """
        detection des zones d"activité normal emg
         voici les on pour l"emg
        #            on1=beg
        #            on2=beg+(12/NORMAL_STANCE_PHASE)*(fo-beg)
        #            on3=beg+(48/NORMAL_STANCE_PHASE)*(fo-beg)
        #            on4=fo+((70-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)
        #            on5=fo+((93-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)
        #            on6=fo+((100-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

        # valeurs normales issues d"une livre EMG analys
        """

        normalActivations = files.openJson(pyCGM2.NORMATIVE_DATABASE_PATH+"emg\\","normalActivation.json")

        NORMAL_STANCE_PHASE=normalActivations["NORMAL_STANCE_PHASE"]
        TABLE = normalActivations["Activation"]


        beg=  (begin-ff)*apf
        fo=(fo-ff)*apf
        end=(end-ff)*apf

        if muscle in TABLE.keys():
            list_beginBurst=[]
            list_burstDuration=[]

            for i,j in zip(range(0,6,2),range(1,6,2)):
                if TABLE[muscle][i]!="na" and  TABLE[muscle][j]!="na":
                    if TABLE[muscle][i]<NORMAL_STANCE_PHASE:
                        beginBurst=beg+(TABLE[muscle][i]/NORMAL_STANCE_PHASE)*(fo-beg)
                    else:
                        beginBurst=fo+((TABLE[muscle][i]-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

                    if TABLE[muscle][j]<NORMAL_STANCE_PHASE:
                        endBurst=beg+(TABLE[muscle][j]/NORMAL_STANCE_PHASE)*(fo-beg)
                    else:
                        endBurst=fo+((TABLE[muscle][j]-NORMAL_STANCE_PHASE)/(100.0-NORMAL_STANCE_PHASE))*(end-fo)

                    list_beginBurst.append(beginBurst)
                    list_burstDuration.append(endBurst-beginBurst)

        else:
            list_beginBurst=[0]
            list_burstDuration=[0]

        return list_beginBurst,list_burstDuration



# -*- coding: utf-8 -*-
from setuptools import setup,find_packages
import os,sys
import string
import logging

#logging.basicConfig(filename = "installer.log", level=logging.DEBUG)

import shutil
import site

developMode = False
if len(sys.argv) >= 2:
    if sys.argv[1] == "develop": developMode = True
if developMode:
    logging.warning("You have sleected a developer model ( local install)")


if sys.maxsize > 2**32:
    raise Exception ("64-bit python version detected. PyCGM2 requires a 32 bits python version")

VERSION ="3.4.4"


for it in site.getsitepackages():
    if "site-packages" in it:
        SITE_PACKAGE_PATH = it +"\\"

NAME_IN_SITEPACKAGE = "pyCGM2-"+VERSION+"-py2.7.egg"


MAIN_PYCGM2_PATH = os.getcwd() + "\\"


PYCGM2_SETTINGS_FOLDER = MAIN_PYCGM2_PATH+"PyCGM2\\Settings\\"
NEXUS_PYCGM2_VST_PATH = MAIN_PYCGM2_PATH + "PyCGM2\\Install\\vst\\"
NEXUS_PIPELINE_TEMPLATE_PATH = MAIN_PYCGM2_PATH + "PyCGM2\\Install\\pipelineTemplate\\"

PATH_TO_PYTHON_SCRIPTS = os.path.dirname(sys.executable)+"\\Scripts\\"

# do not serve anymore since all apps are now in Scripts ( i still keep it)
if not developMode:
    PATH_IN_SITEPACKAGE = SITE_PACKAGE_PATH+NAME_IN_SITEPACKAGE+"\\"
else:
    PATH_IN_SITEPACKAGE = MAIN_PYCGM2_PATH

user_folder =  os.getenv("PUBLIC")
NEXUS_PUBLIC_PATH = user_folder+"\\Documents\\Vicon\\Nexus2.x\\"
NEXUS_PUBLIC_DOCUMENT_VST_PATH = NEXUS_PUBLIC_PATH + "ModelTemplates\\"
NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH = NEXUS_PUBLIC_PATH+"Configurations\\Pipelines\\"

def parse_requirements(requirements):
    with open(requirements) as f:
        return [l.strip('\n') for l in f if l.strip('\n') and not l.startswith('#')]

reqs = parse_requirements("requirements.txt")

def scanViconTemplatePipeline(sourcePath,desPath,pyCGM2nexusAppsPath):

    toreplace= "[TOREPLACE]"

    sourcePath = sourcePath[:-1] if sourcePath[-1:]=="\\" else sourcePath
    desPath = desPath[:-1] if desPath[-1:]=="\\" else desPath
    pyCGM2nexusAppsPath = pyCGM2nexusAppsPath[:-1] if pyCGM2nexusAppsPath[-1:]=="\\" else pyCGM2nexusAppsPath

    pyCGM2nexusAppsPath_antislash = string.replace(pyCGM2nexusAppsPath, '\\', '/')

    for file in os.listdir(sourcePath):
        with open(sourcePath+"\\"+file, 'r') as f:
            file_contents = f.read()

        content = string.replace(file_contents, toreplace,pyCGM2nexusAppsPath_antislash)


        if not os.path.isfile( desPath +"\\"+ file):
            with open(desPath + "\\"+file, "w") as text_file:
                text_file.write(content)


def gen_data_files(*dirs):
    results = []
    for src_dir in dirs:
        for root,dirs,files in os.walk(src_dir):
            results.append((root, map(lambda f:root + "/" + f, files)))
    return results

def gen_data_files_forScripts(*dirs):
    results = []
    for src_dir in dirs:
        for root,dirs,files in os.walk(src_dir):
            for file in files:
                if file[-3:] ==".py":
                    results.append(os.path.join(root, file))
    return results




def getSubDirectories(dir):
    subdirs = [d for d in os.listdir(dir) if os.path.isdir(os.path.join(dir, d))]
    return subdirs

def getFiles(dir):
    return  [f for f in os.listdir(dir) if os.path.isfile(os.path.join(dir, f))]

#------------------------- UNINSTALL--------------------------------------------

# remove pyCGM2 folder or egg-link
dirSitepackage = getSubDirectories(SITE_PACKAGE_PATH[:-1])
for folder in  dirSitepackage:
    if "pyCGM2" in folder:
        shutil.rmtree(SITE_PACKAGE_PATH+folder)
        logging.info("package pyCGM2 (%s) removed"%(folder))



if "pyCGM2.egg-link" in os.listdir(SITE_PACKAGE_PATH[:-1]):
    os.remove(SITE_PACKAGE_PATH+"pyCGM2.egg-link")

# remove Build/dist/egg info in the downloaded folder
localDirPath = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__))))
localDirPathDirs = getSubDirectories(localDirPath)
if "build" in  localDirPathDirs:    shutil.rmtree(localDirPath+"\\build")
if "dist" in  localDirPathDirs:     shutil.rmtree(localDirPath+"\\dist")
if "pyCGM2.egg-info" in  localDirPathDirs:     shutil.rmtree(localDirPath+"\\pyCGM2.egg-info")


# delete everything in programData
if os.getenv("PROGRAMDATA") is not None:
    pd = os.getenv("PROGRAMDATA")
    pddirs = getSubDirectories(pd)
    if "pyCGM2" in  pddirs:
        shutil.rmtree(pd+"\\pyCGM2")
        logging.info("pprogramData/pyCGM2---> remove")

if os.path.isdir(NEXUS_PUBLIC_PATH):
    # delete all previous vst and pipelines in Nexus Public Documents
    files = getFiles(NEXUS_PUBLIC_DOCUMENT_VST_PATH)
    for file in files:
        if "pyCGM2" in file[0:6]: # check 6 first letters
            os.remove(os.path.join(NEXUS_PUBLIC_DOCUMENT_VST_PATH,file))

    files = getFiles(NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH)
    for file in files:
        if "pyCGM2" in file[0:6]:
            os.remove(os.path.join(NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH,file))


# dirs = getSubDirectories(NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH)
# if "pyCGM2" in dirs:
#     shutil.rmtree(NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH+"pyCGM2")
#------------------------------------------------------------------


#------------------------- PRE INSTALL---------------------------------------

#--- management of the folder ProgramData/pyCGM2----
if not developMode:
    if os.getenv("PROGRAMDATA"):
        PYCGM2_APPDATA_PATH = os.getenv("PROGRAMDATA")+"\\pyCGM2"
        shutil.copytree(PYCGM2_SETTINGS_FOLDER[:-1], PYCGM2_APPDATA_PATH)

#--- management of nexus-related files ( vst+pipelines)-----
if os.path.isdir(NEXUS_PUBLIC_PATH):
    # vst
    content = os.listdir(NEXUS_PYCGM2_VST_PATH[:-1])
    for item in content:
        full_filename = os.path.join(NEXUS_PYCGM2_VST_PATH, item)
        shutil.copyfile(full_filename,  os.path.join(NEXUS_PUBLIC_DOCUMENT_VST_PATH,item))


    scanViconTemplatePipeline(NEXUS_PIPELINE_TEMPLATE_PATH,
                                                NEXUS_PUBLIC_DOCUMENT_PIPELINE_PATH,
                                                PATH_TO_PYTHON_SCRIPTS)

else:
    logging.error("[pyCGM2] - Nexus folder not detected - No generation of VST and pipelines")

#------------------------- INSTALL--------------------------------------------
# numpy 1.11 scipy==1.2.1'matplotlib<3.0.0' pandas ==0.19.1'enum34>=1.1.2 configparser>=3.5.0' beautifulsoup4>=3.5.0' 'pyyaml>=3.13.0 yamlordereddictloader>=0.4.0 'xlrd >=0.9.0
setup(name = 'pyCGM2',
    version = VERSION,
    author = 'Fabien Leboeuf',
    author_email = 'fabien.leboeuf@gmail.com',
    description = "Conventional Gait models and Gait analysis",
    long_description= "A python implementation of conventional gait models and methods for processing motion capture data",
    url = 'https://pycgm2.github.io',
    keywords = 'python CGM Vicon PluginGait',
    packages=find_packages(),
	include_package_data=True,
    license='CC-BY-SA',
	install_requires = reqs,
    #'qtmWebGaitReport>=0.0.1'],
    classifiers=['Programming Language :: Python',
                 'Programming Language :: Python :: 2.7',
                 'Operating System :: Microsoft :: Windows',
                 'Natural Language :: English'],
    #scripts=gen_data_files_forScripts("Apps/ViconApps")
    entry_points={
          'console_scripts': [
                # NEXUS
                'Nexus_CGM1_Calibration  =  pyCGM2.Apps.ViconApps.CGM1.CGM1_Calibration:main',
                'Nexus_CGM1_Fitting      =  pyCGM2.Apps.ViconApps.CGM1.CGM1_Fitting:main',
                'Nexus_CGM11_Calibration =  pyCGM2.Apps.ViconApps.CGM1_1.CGM1_1_Calibration:main',
                'Nexus_CGM11_Fitting     =  pyCGM2.Apps.ViconApps.CGM1_1.CGM1_1_Fitting:main',
                'Nexus_CGM21_Calibration =  pyCGM2.Apps.ViconApps.CGM2_1.CGM2_1_Calibration:main',
                'Nexus_CGM21_Fitting     =  pyCGM2.Apps.ViconApps.CGM2_1.CGM2_1_Fitting:main',
                'Nexus_CGM22_Calibration =  pyCGM2.Apps.ViconApps.CGM2_2.CGM2_2_Calibration:main',
                'Nexus_CGM22_Fitting     =  pyCGM2.Apps.ViconApps.CGM2_2.CGM2_2_Fitting:main',
                'Nexus_CGM23_Calibration =  pyCGM2.Apps.ViconApps.CGM2_3.CGM2_3_Calibration:main',
                'Nexus_CGM23_Fitting     =  pyCGM2.Apps.ViconApps.CGM2_3.CGM2_3_Fitting:main',
                'Nexus_CGM24_Calibration =  pyCGM2.Apps.ViconApps.CGM2_4.CGM2_4_Calibration:main',
                'Nexus_CGM24_Fitting     =  pyCGM2.Apps.ViconApps.CGM2_4.CGM2_4_Fitting:main',
                'Nexus_CGM25_Calibration =  pyCGM2.Apps.ViconApps.CGM2_5.CGM2_5_Calibration:main',
                'Nexus_CGM25_Fitting     =  pyCGM2.Apps.ViconApps.CGM2_5.CGM2_5_Fitting:main',
                'Nexus_CGM26_2DOF =  pyCGM2.Apps.ViconApps.CGM2_6.CGM_Knee2DofCalibration:main',
                'Nexus_CGM26_SARA     =  pyCGM2.Apps.ViconApps.CGM2_6.CGM_Knee2DofCalibration:main',

                'Nexus_plotMAP                      =  pyCGM2.Apps.ViconApps.DataProcessing.plotMAP:main',
                'Nexus_plotNormalizedKinematics     =  pyCGM2.Apps.ViconApps.DataProcessing.plotNormalizedKinematics:main',
                'Nexus_plotNormalizedKinetics       =  pyCGM2.Apps.ViconApps.DataProcessing.plotNormalizedKinetics:main',
                'Nexus_plotSpatioTemporalParameters =  pyCGM2.Apps.ViconApps.DataProcessing.plotSpatioTemporalParameters:main',
                'Nexus_plotTemporalKinematics       =  pyCGM2.Apps.ViconApps.DataProcessing.plotTemporalKinematics:main',
                'Nexus_plotTemporalKinetics         =  pyCGM2.Apps.ViconApps.DataProcessing.plotTemporalKinetics:main',

                'Nexus_plotNormalizedEmg = pyCGM2.Apps.ViconApps.EMG.plotNormalizedEmg:main',
                'Nexus_plotTemporalEmg   = pyCGM2.Apps.ViconApps.EMG.plotTemporalEmg:main',

                'Nexus_zeniDetector     =  pyCGM2.Apps.ViconApps.Events.zeniDetector:main',
                'Nexus_KalmanGapFilling =  pyCGM2.Apps.ViconApps.MoGapFill.KalmanGapFilling:main',

                'Nexus_resetProgramData =  pyCGM2.Apps.ViconApps.Miscellaneous.pyCGM2_resetProgramData:main',
                'Nexus_check_inputArgs  =  pyCGM2.Apps.ViconApps.Miscellaneous.check_inputArgs:main',

                'pyCGM2-copyPasteCgmSettings  =  pyCGM2.Apps.Commands.commands:copyPasteCgmSettings',
                'pyCGM2-copyPasteEmgSettings  =  pyCGM2.Apps.Commands.commands:copyPasteEmgSettings',


                # QTM
                'QTM_CGM1_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM1_workflow:command',
                'QTM_CGM11_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM11_workflow:command',
                'QTM_CGM21_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM21_workflow:command',
                'QTM_CGM22_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM22_workflow:command',
                'QTM_CGM23_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM23_workflow:command',
                'QTM_CGM24_workflow  =  pyCGM2.Apps.QtmApps.CGMi.CGM24_workflow:command',

          ]
      },
    )



import sys
import glob
import re
import os
import logging

NEXUS_VERSION = None # ex: "Nexus2.9" force the use of version 2.9

def getLastNexusVersion():
    nexusDir = "C:\Program Files (x86)\Vicon"
    dirs = os.listdir(nexusDir)
    li =[]
    for it in dirs:
        if "Nexus2" in it:
            version = int(it[it.find(".")+1:])
            li.append(version)
    last = max(li)
    return "Nexus2."+str(last)

try:
    if NEXUS_VERSION is None:
        NEXUS_VERSION = getLastNexusVersion()

    version = float(NEXUS_VERSION[5:])
    if version<2.12:
        if not "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Python" in sys.path:
            sys.path.append( "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Python")

        if not "C:/Program Files (x86)/Vicon/Nexus"+NEXUS_VERSION+"/SDK/Win32" in sys.path:
            sys.path.append( "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Win32")
    else :
        if not "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Win32/Python/viconnexusapi" in sys.path:
            sys.path.append( "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Win32/Python/viconnexusapi")

        if not "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Win32/Python/viconnexusutils" in sys.path:
            sys.path.append( "C:/Program Files (x86)/Vicon/"+NEXUS_VERSION+"/SDK/Win32/Python/viconnexusutils")

except Exception, errormsg:
    logging.info (errormsg)


ENCODER = "latin-1"

# CONSTANTS
MAIN_PYCGM2_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir)) + "\\"

#  [Optional] setting folder
PYCGM2_SETTINGS_FOLDER = MAIN_PYCGM2_PATH+"pyCGM2\Settings\\"


#  [Optional]programData
if  os.path.isdir(os.getenv("PROGRAMDATA")+"\\pyCGM2"):
    PYCGM2_APPDATA_PATH = os.getenv("PROGRAMDATA")+"\\pyCGM2\\"
else:
    PYCGM2_APPDATA_PATH = PYCGM2_SETTINGS_FOLDER




# [Optional]: Apps path
MAIN_PYCGM2_APPS_PATH = MAIN_PYCGM2_PATH+"Apps\\"

# [Optional] path to embbbed Normative data base.
NORMATIVE_DATABASE_PATH = MAIN_PYCGM2_PATH +"pyCGM2\\Data\\normativeData\\"  # By default, use pyCGM2-embedded normative data ( Schartz - Pinzone )

# [Optional] main folder containing osim model
OPENSIM_PREBUILD_MODEL_PATH = PYCGM2_APPDATA_PATH + "opensim\\"

# [Optional] path pointing at Data Folders used for Tests

TEST_DATA_PATH = "C:\\Users\\fleboeuf\\Documents\\DATA\\pyCGM2-Data-Tests\\"
TEST_DATA_PATH_OUT = "C:\\Users\\fleboeuf\\Documents\\DATA\\pyCGM2-Data-Tests-OUT\\"
MAIN_BENCHMARK_PATH = "C:\\Users\\fleboeuf\\Documents\\DATA\\Gait patterns\\"

# [optional] path pointing pyCGM2-Nexus tools
NEXUS_PYCGM2_TOOLS_PATH = MAIN_PYCGM2_PATH + "pyCGM2\\Nexus\\"

PYCGM2_SCRIPTS_PATH = MAIN_PYCGM2_PATH+"Scripts\\"



# -*- coding: utf-8 -*-
import pyCGM2
from viconnexusapi import ViconNexus

import numpy as np
import logging

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

NEXUS = ViconNexus.ViconNexus()

class Channel(object):

    def __init__(self, label,values,unit,description):
        self.m_label = label
        self.m_values = values
        self.m_description = description
        self.m_unit = unit

    def getLabel(self):
        return  self.m_label
    def getValues(self):
        return self.m_values
    def getDescription(self):
        return  self.m_description
    def getUnit(self):
        return self.m_unit



class Device(object):

    def __init__(self, id):
        self.m_id = id

        self.m_name = NEXUS.GetDeviceDetails(self.m_id)[0]
        self.m_type = NEXUS.GetDeviceDetails(self.m_id)[1]
        self.m_frequency = NEXUS.GetDeviceDetails(self.m_id)[2]
        self.m_outputIds = NEXUS.GetDeviceDetails(self.m_id)[3]

        self.m_forcePlateInfo = NEXUS.GetDeviceDetails(self.m_id)[4]
        self.m_eyeTrackerInfo = NEXUS.GetDeviceDetails(self.m_id)[5]


    def getDeviceName(self):
        return self.m_name

    def getDeviceFrequency(self):
        return self.m_frequency

    def getOutputNames(self):
        out = list()
        for i in self.m_outputIds:
            out.append(NEXUS.GetDeviceOutputDetails(self.m_id,i)[0])
        return out



class AnalogDevice(Device):

    def __init__(self, id):
        super(AnalogDevice, self).__init__(id)
        self.m_channels = list()
        #self.m_id = id

    def getUnit(self):
        unit = list()
        for outputId in self.m_outputIds:
            unit.append(str(NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[2]))
        return  unit


    def getChannels(self):

        for outputId in self.m_outputIds:
            outputName = NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]
            outputType = NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[1]
            unit = str(NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[2])
            i=0
            for channelId in NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[5]:
                label = str(outputName) +"."+ str(NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4][i])
                values = np.asarray(NEXUS.GetDeviceChannel(self.m_id,outputId,channelId)[0])
                description =  "Analog::"+str(outputType)+" ["+str(self.m_id) + "," + str(channelId)+"]"
                self.m_channels.append(Channel(label,values,unit,description))
                i+=1

        return self.m_channels



class ForcePlate(Device):

    def __init__(self, id):
        super(ForcePlate, self).__init__(id)


    def getDescription(self):
            return str("Force Plate [" +str(self.m_id) + "]")
#
    def getOrigin(self):
        nfp_info = self.m_forcePlateInfo
        return np.asarray(nfp_info.WorldT)

    def getLocalOrigin(self):
        nfp_info = self.m_forcePlateInfo
        return np.asarray(nfp_info.LocalT)


    def getContext(self):
        nfp_info = self.m_forcePlateInfo
        return nfp_info.Context

    def getPhysicalOrigin(self):
        nfp_info = self.m_forcePlateInfo
        worldR =  np.asarray(nfp_info.WorldR).reshape((3,3))
        origin = self.getOrigin()
        return origin + np.dot(worldR, np.asarray(nfp_info.LocalT))



    def getOrientation(self):
        nfp_info = self.m_forcePlateInfo
        worldR =  np.asarray(nfp_info.WorldR).reshape((3,3))
        localR =  np.asarray(nfp_info.LocalR).reshape((3,3))

        return np.dot(worldR,localR)

    def getForceUnit(self):

        for outputId in self.m_outputIds:
            if NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4] == ["Fx","Fy","Fz"]:
                outputName =  NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]

        deviceOutputID = NEXUS.GetDeviceOutputIDFromName(self.m_id,outputName)
        return str(NEXUS.GetDeviceOutputDetails(self.m_id,deviceOutputID)[2])

    def getMomentUnit(self):

        for outputId in self.m_outputIds:
            if NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4] == ["Mx","My","Fz"]:
                outputName =  NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]


        deviceOutputID = NEXUS.GetDeviceOutputIDFromName(self.m_id,outputName)
        return str(NEXUS.GetDeviceOutputDetails(self.m_id,deviceOutputID)[2])



    def getGlobalForce(self):

        for outputId in self.m_outputIds:
            if NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4] == ["Fx","Fy","Fz"]:
                outputName =  NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]

        deviceOutputID = NEXUS.GetDeviceOutputIDFromName(self.m_id,outputName)

        channelID_x = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Fx')
        channelID_y = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Fy')
        channelID_z = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Fz')

        x = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_x )[0])
        y = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_y )[0])
        z = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_z )[0])

        return np.array([x,y,z]).T


    def getGlobalMoment(self):

        for outputId in self.m_outputIds:
            if NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4] == ["Mx","My","Mz"]:
                outputName =  NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]

        deviceOutputID = NEXUS.GetDeviceOutputIDFromName(self.m_id,outputName)

        channelID_x = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Mx')
        channelID_y = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'My')
        channelID_z = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Mz')

        x = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_x )[0])
        y = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_y )[0])
        z = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_z )[0])

        return np.array([x,y,z]).T


    def getGlobalCoP(self):

        for outputId in self.m_outputIds:
            if NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[4] == ["Cx","Cy","Cz"]:
                outputName =  NEXUS.GetDeviceOutputDetails(self.m_id,outputId)[0]

        deviceOutputID = NEXUS.GetDeviceOutputIDFromName(self.m_id,outputName)

        channelID_x = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Cx')
        channelID_y = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Cy')
        channelID_z = NEXUS.GetDeviceChannelIDFromName(self.m_id, deviceOutputID, 'Cz')

        x = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_x )[0])
        y = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_y )[0])
        z = np.asarray(NEXUS.GetDeviceChannelGlobal( self.m_id, deviceOutputID, channelID_z )[0])

        return np.array([x,y,z]).T


    def getLocalReactionForce(self):

        force = -1* self.getGlobalForce()
        R = self.getOrientation()

        out = np.zeros(force.shape)
        for i in range(0, force.shape[0]):
            out[i,:] = np.dot(R.T,force[i,:])
        return out

    def getLocalReactionMoment(self):

        moment = self.getGlobalMoment()
        force = self.getGlobalForce()
        #origin = -1*self.getLocalPhysicOrigin()

        origin = self.getOrigin() - self.getPhysicalOrigin()

        moment_transp = np.zeros(moment.shape)
        moment_transp[:,0] =  moment[:,0] - (force[:,1] *  origin[2] - origin[1]*force[:,2])
        moment_transp[:,1] =  moment[:,1] - (force[:,2] *  origin[0] - origin[2]*force[:,0])
        moment_transp[:,2] =  moment[:,2] - (force[:,0] *  origin[1] - origin[0]*force[:,1])

        moment = -1* moment_transp#self.getGlobalMoment()
        R = self.getOrientation()



        out = np.zeros(moment.shape)
        for i in range(0, moment.shape[0]):
            out[i,:] = np.dot(R.T,moment[i,:])
        return out



    def getCorners(self):
        nfp_info = self.m_forcePlateInfo


        R = np.asarray(nfp_info.WorldR).reshape((3,3))
        t = np.asarray(nfp_info.WorldT)

        corner1 = np.asarray(nfp_info.UpperBounds)
        corner3 = np.asarray(nfp_info.LowerBounds)
        corner2 = np.asarray(nfp_info.UpperBounds)*np.array([1,-1,0])
        corner0 = np.asarray(nfp_info.LowerBounds)*np.array([1,-1,0])

        corners = np.zeros((3,4))
        corners[:,0] = np.dot(R,corner0)+t #2
        corners[:,1] = np.dot(R,corner1)+t
        corners[:,2] = np.dot(R,corner2)+t
        corners[:,3] = np.dot(R,corner3)+t

        return corners



# coding: utf-8
import cPickle
import logging
import json
import os
from shutil import copyfile
from collections import OrderedDict
import shutil
import yaml
import yamlordereddictloader
from bs4 import BeautifulSoup
from datetime import datetime
import pandas as pd
import pyCGM2


def openFile(path,filename):
    """

    """
    if os.path.isfile( (path + filename)):
        content = open((path+filename)).read()

        jsonFlag = is_json(content)
        yamlFlag = is_yaml(content)
        if jsonFlag:
            logging.debug("your file (%s) matches json syntax"%filename)
            struct = openJson(path ,(filename))

        if yamlFlag:
            logging.debug("your file (%s) matches yaml syntax"%filename)
            struct = openYaml(path,filename)

        if not yamlFlag and not yamlFlag:
            raise Exception ("%s is neither a Yaml or a json file"%filename)

        return struct
    else:
        return False

def readContent(stringContent):

    jsonFlag = is_json(stringContent)
    yamlFlag = is_yaml(stringContent)
    if jsonFlag:
        logging.debug("your content  matches json syntax")
        struct = json.loads(stringContent,object_pairs_hook=OrderedDict)

    if yamlFlag:
        logging.debug("your content  matches yaml syntax")
        struct =  yaml.load(stringContent,Loader=yamlordereddictloader.Loader)

    if not yamlFlag and not yamlFlag:
        raise Exception ("content is neither a Yaml or a json")

    return struct



def loadModel(path,FilenameNoExt):
    if FilenameNoExt is not None:
        filename = FilenameNoExt + "-pyCGM2.model"
    else:
        filename = "pyCGM2.model"

    # --------------------pyCGM2 MODEL ------------------------------
    if not os.path.isfile((path + filename)):
        raise Exception ("%s-pyCGM2.model file doesn't exist. Run CGM Calibration operation"%filename)
    else:
        f = open((path+filename), 'rb')
        model = cPickle.load(f)
        f.close()

        return model

def saveModel(model,path,FilenameNoExt):

    if FilenameNoExt is not None:
        filename = FilenameNoExt + "-pyCGM2.model"
    else:
        filename = "pyCGM2.model"

    #pyCGM2.model
    if os.path.isfile((path + filename)):
        logging.warning("previous model removed")
        os.remove((path + filename))

    modelFile = open((path+filename), "wb")
    cPickle.dump(model, modelFile,-1)
    modelFile.close()


def loadAnalysis(path,FilenameNoExt):
    if FilenameNoExt is not None:
        filename = FilenameNoExt + "-pyCGM2.analysis"
    else:
        filename = "pyCGM2.analysis"

    # --------------------pyCGM2 MODEL ------------------------------
    if not os.path.isfile((path + filename)):
        raise Exception ("%s-pyCGM2.analysis file doesn't exist"%filename)
    else:
        f = open((path+filename), 'rb')
        analysis = cPickle.load(f)
        f.close()

        return analysis

def saveAnalysis(analysisInstance,path,FilenameNoExt):

    if FilenameNoExt is not None:
        filename = FilenameNoExt + "-pyCGM2.analysis"
    else:
        filename = "pyCGM2.analysis"

    #pyCGM2.model
    if os.path.isfile((path + filename)):
        logging.warning("previous analysis removed")
        os.remove((path + filename))

    analysisFile = open((path+filename), "wb")
    cPickle.dump(analysisInstance, analysisFile,-1)
    analysisFile.close()


def openJson(path,filename):

    if path is not None: path = path
    filename = filename

    try:
        if path is None:
            jsonStuct= json.loads(open((filename)).read(),object_pairs_hook=OrderedDict)
        else:
            jsonStuct= json.loads(open((path+filename)).read(),object_pairs_hook=OrderedDict)
        return jsonStuct
    except :
        raise Exception ("[pyCGM2] : json syntax of file (%s) is incorrect. check it" %(filename))

def saveJson(path, filename, content):
    if path is not None: path = path
    filename = filename
    if path is None:
        with open((filename), 'w') as outfile:
            json.dump(content, outfile,indent=4)
    else:
        with open((path+filename), 'w') as outfile:
            json.dump(content, outfile,indent=4)


def prettyDictPrint(parsedContent):
    print (json.dumps(parsedContent, indent=4, sort_keys=True))



def openYaml(path,filename):
    if path is not None: path = path
    filename = filename
    try:
        if path is None:
            struct = yaml.load(open((filename)).read(),Loader=yamlordereddictloader.Loader)
        else:
            struct= yaml.load(open((path+filename)).read(),Loader=yamlordereddictloader.Loader)
        return struct
    except :
        raise Exception ("[pyCGM2] : yaml syntax of file (%s) is incorrect. check it" %(filename))



def getTranslators(DATA_PATH, translatorType = "CGM1.translators"):
    #  translators management
    if os.path.isfile( (DATA_PATH + translatorType)):
       logging.warning("local translator found")

       sessionTranslators = openFile(DATA_PATH,translatorType)
       translators = sessionTranslators["Translators"]
       return translators
    else:
       return False

def getIKweightSet(DATA_PATH, ikwf):
    #  translators management
    if os.path.isfile( (DATA_PATH + ikwf)):
       logging.warning("local ik weightSet file found")
       ikWeight = openFile(DATA_PATH,ikwf)
       return ikWeight
    else:
       return False

def getMpFileContent(DATA_PATH,file,subject):

    if subject is not None:
        out = subject+"-" + file
    else:
        out = file

    if not os.path.isfile( (DATA_PATH + file)):
        copyfile((pyCGM2.PYCGM2_SETTINGS_FOLDER+file), (DATA_PATH + out))
        logging.warning("Copy of %s from pyCGM2 Settings folder"%(file))

    content = openFile(DATA_PATH,out)

    return content,out

def getMp(mpInfo,resetFlag=True):

    required_mp={
    'Bodymass'   : mpInfo["MP"]["Required"]["Bodymass"],
    'Height'   : mpInfo["MP"]["Required"]["Height"],
    'LeftLegLength' :mpInfo["MP"]["Required"]["LeftLegLength"],
    'RightLegLength' : mpInfo["MP"]["Required"][ "RightLegLength"],
    'LeftKneeWidth' : mpInfo["MP"]["Required"][ "LeftKneeWidth"],
    'RightKneeWidth' : mpInfo["MP"]["Required"][ "RightKneeWidth"],
    'LeftAnkleWidth' : mpInfo["MP"]["Required"][ "LeftAnkleWidth"],
    'RightAnkleWidth' : mpInfo["MP"]["Required"][ "RightAnkleWidth"],
    'LeftSoleDelta' : mpInfo["MP"]["Required"][ "LeftSoleDelta"],
    'RightSoleDelta' : mpInfo["MP"]["Required"]["RightSoleDelta"],
    'LeftShoulderOffset' : mpInfo["MP"]["Required"]["LeftShoulderOffset"],
    'RightShoulderOffset' : mpInfo["MP"]["Required"]["RightShoulderOffset"],
    'LeftElbowWidth' : mpInfo["MP"]["Required"]["LeftElbowWidth"],
    'LeftWristWidth' : mpInfo["MP"]["Required"]["LeftWristWidth"],
    'LeftHandThickness' : mpInfo["MP"]["Required"]["LeftHandThickness"],
    'RightElbowWidth' : mpInfo["MP"]["Required"]["RightElbowWidth"],
    'RightWristWidth' : mpInfo["MP"]["Required"]["RightWristWidth"],
    'RightHandThickness' : mpInfo["MP"]["Required"]["RightHandThickness"]

    }

    if resetFlag:
        optional_mp={
        'InterAsisDistance'   : 0,
        'LeftAsisTrocanterDistance' : 0,
        'LeftTibialTorsion' : 0 ,
        'LeftThighRotation' : 0,
        'LeftShankRotation' : 0,
        'RightAsisTrocanterDistance' : 0,
        'RightTibialTorsion' :0 ,
        'RightThighRotation' : 0,
        'RightShankRotation' : 0
        }
    else:
        optional_mp={
        'InterAsisDistance'   : mpInfo["MP"]["Optional"][ "InterAsisDistance"],#0,
        'LeftAsisTrocanterDistance' : mpInfo["MP"]["Optional"][ "LeftAsisTrocanterDistance"],#0,
        'LeftTibialTorsion' : mpInfo["MP"]["Optional"][ "LeftTibialTorsion"],#0 ,
        'LeftThighRotation' : mpInfo["MP"]["Optional"][ "LeftThighRotation"],#0,
        'LeftShankRotation' : mpInfo["MP"]["Optional"][ "LeftShankRotation"],#0,
        'RightAsisTrocanterDistance' : mpInfo["MP"]["Optional"][ "RightAsisTrocanterDistance"],#0,
        'RightTibialTorsion' : mpInfo["MP"]["Optional"][ "RightTibialTorsion"],#0 ,
        'RightThighRotation' : mpInfo["MP"]["Optional"][ "RightThighRotation"],#0,
        'RightShankRotation' : mpInfo["MP"]["Optional"][ "RightShankRotation"],#0,
        }

    return required_mp,optional_mp



def saveMp(mpInfo,model,DATA_PATH,mpFilename):

    # update optional mp and save a new info file
    mpInfo["MP"]["Required"][ "Bodymass"] = model.mp["Bodymass"]
    mpInfo["MP"]["Required"][ "LeftLegLength"] = model.mp["LeftLegLength"]
    mpInfo["MP"]["Required"][ "RightLegLength"] = model.mp["RightLegLength"]
    mpInfo["MP"]["Required"][ "LeftKneeWidth"] = model.mp["LeftKneeWidth"]
    mpInfo["MP"]["Required"][ "RightKneeWidth"] = model.mp["RightKneeWidth"]
    mpInfo["MP"]["Required"][ "LeftAnkleWidth"] = model.mp["LeftAnkleWidth"]
    mpInfo["MP"]["Required"][ "RightAnkleWidth"] = model.mp["RightAnkleWidth"]
    mpInfo["MP"]["Required"][ "LeftSoleDelta"] = model.mp["LeftSoleDelta"]
    mpInfo["MP"]["Required"][ "RightSoleDelta"] = model.mp["RightSoleDelta"]
    mpInfo["MP"]["Required"][ "LeftShoulderOffset"] = model.mp["LeftShoulderOffset"]
    mpInfo["MP"]["Required"][ "RightShoulderOffset"] = model.mp["RightShoulderOffset"]
    mpInfo["MP"]["Required"][ "LeftElbowWidth"] = model.mp["LeftElbowWidth"]
    mpInfo["MP"]["Required"][ "LeftWristWidth"] = model.mp["LeftWristWidth"]
    mpInfo["MP"]["Required"][ "LeftHandThickness"] = model.mp["LeftHandThickness"]
    mpInfo["MP"]["Required"][ "RightElbowWidth"] = model.mp["RightElbowWidth"]
    mpInfo["MP"]["Required"][ "RightWristWidth"] = model.mp["RightWristWidth"]
    mpInfo["MP"]["Required"][ "RightHandThickness"] = model.mp["RightHandThickness"]


    mpInfo["MP"]["Optional"][ "InterAsisDistance"] = model.mp_computed["InterAsisDistance"]
    mpInfo["MP"]["Optional"][ "LeftAsisTrocanterDistance"] = model.mp_computed["LeftAsisTrocanterDistance"]
    mpInfo["MP"]["Optional"][ "LeftTibialTorsion"] = model.mp_computed["LeftTibialTorsionOffset"]
    mpInfo["MP"]["Optional"][ "LeftThighRotation"] = model.mp_computed["LeftThighRotationOffset"]
    mpInfo["MP"]["Optional"][ "LeftShankRotation"] = model.mp_computed["LeftShankRotationOffset"]

    mpInfo["MP"]["Optional"][ "RightAsisTrocanterDistance"] = model.mp_computed["RightAsisTrocanterDistance"]
    mpInfo["MP"]["Optional"][ "RightTibialTorsion"] = model.mp_computed["RightTibialTorsionOffset"]
    mpInfo["MP"]["Optional"][ "RightThighRotation"] = model.mp_computed["RightThighRotationOffset"]
    mpInfo["MP"]["Optional"][ "RightShankRotation"] = model.mp_computed["RightShankRotationOffset"]

    mpInfo["MP"]["Optional"][ "LeftKneeFuncCalibrationOffset"] = model.mp_computed["LeftKneeFuncCalibrationOffset"]
    mpInfo["MP"]["Optional"][ "RightKneeFuncCalibrationOffset"] = model.mp_computed["RightKneeFuncCalibrationOffset"]

    saveJson(DATA_PATH, mpFilename, mpInfo)



def getFiles(path, extension, ignore=None):

    out=list()
    for file in os.listdir(path):
        if ignore is None:
            if file.endswith(extension):
                out.append(file)
        else:
            if file.endswith(extension) and ignore not in file:
                out.append(file)

    return out


def getC3dFiles(path, text="", ignore=None ):

    out=list()
    for file in os.listdir(path):
       if ignore is None:
           if file.endswith(".c3d"):
               if text in file:  out.append(file)
       else:
           if file.endswith(".c3d") and ignore not in file:
               if text in file:  out.append(file)

    return out

def copySessionFolder(folderPath, folder2copy, newFolder, selectedFiles=None):

    if not os.path.isdir((folderPath+"\\"+newFolder)):
        os.makedirs((folderPath+"\\"+newFolder))

    for file in os.listdir((folderPath+"\\"+folder2copy)):
        if file.endswith(".Session.enf"):
            src = (folderPath+"\\"+folder2copy+"\\" +file)
            dst = (folderPath+"\\"+newFolder+"\\" +newFolder+".Session.enf")

            shutil.copyfile(src, dst)
        else:
            if selectedFiles is None:
                fileToCopy = file
                src = (folderPath+"\\"+folder2copy+"\\" +fileToCopy)
                dst = (folderPath+"\\"+newFolder+"\\" + fileToCopy)

                shutil.copyfile(src, dst)


            else:
                if file in selectedFiles:
                    fileToCopy = file

                    src = (folderPath+"\\"+folder2copy+"\\" +fileToCopy)
                    dst = (folderPath+"\\"+newFolder+"\\" + fileToCopy)

                    shutil.copyfile(src, dst)

def createDir(fullPathName):
    fullPathName = fullPathName
    pathOut = fullPathName[:-1] if fullPathName[-1:]=="\\" else fullPathName
    if not os.path.isdir((pathOut)):
        os.makedirs((pathOut))
    else:
        logging.warning("directory already exists")

def getDirs(folderPath):
    folderPath = folderPath
    pathOut = folderPath[:-1] if folderPath[-1:]=="\\" else folderPath
    dirs = [ name for name in os.listdir(pathOut) if os.path.isdir(os.path.join(pathOut, name)) ]
    return ( dirs)

def try_as(loader, s, on_error):
    try:
        loader(s)
        return True
    except on_error:
        return False

def is_json(s):
    return try_as(json.loads, s, ValueError)

def is_yaml(s):
    return try_as(yaml.safe_load, s, yaml.scanner.ScannerError)

def copyPaste(src, dst):
    shutil.copyfile(src,
                    dst)

def copyPasteDirectory(src, dst):
    shutil.copytree(src,
                    dst)

def deleteDirectory(dir):
    shutil.rmtree(dir)


def readXml(DATA_PATH,filename):
    infile = open((DATA_PATH+filename),"rb")
    contents = infile.read()
    soup = BeautifulSoup(contents,'xml')

    return soup


def getFileCreationDate(file):
    """
    str(getFileCreationDate(file).date())
    str(getFileCreationDate(file))
    """
    stat = os.stat(file)
    try:
        stamp =  stat.st_birthtime
    except AttributeError:
        stamp= stat.st_mtime

    return datetime.fromtimestamp(stamp)


def concatenateExcelFiles(DATA_PATH_OUT,outputFilename,sheetNames,xlsFiles):

    xlsxWriter = pd.ExcelWriter((DATA_PATH_OUT+outputFilename+".xlsx"))
    for sheet in sheetNames:
        df_total = pd.DataFrame()
        for file in xlsFiles:
            excel_file = pd.ExcelFile(file)
            sheets = excel_file.sheet_names
            if sheet in sheets:
                df = excel_file.parse(sheetname = sheet)
                df_total = df_total.append(df)

        df_total.to_excel(xlsxWriter,sheet,index=False)

    xlsxWriter.save()

def convertPickleToBinary(path,filename):
    
    f = open(path+filename, 'r')
    analysis = cPickle.load(f)
    f.close()

    f2 = open(path+filename,  "wb")   # 'wb' instead 'w' for binary file
    cPickle.dump(analysis, f2, -1)       # -1 specifies highest binary protocol
    f2.close()



from __future__ import print_function
import time


class Timer(object):
    """ A quick tic-toc timer

        def usage1():
        print 'Verbose mode'
        with Timer('Usage 1'):
            # do stuff
            for _ in range(1000000):
                pass
    """

    def __init__(self, name=None, verbose=True):
        self.name = name
        self.verbose = verbose
        self.elapsed = None

    def __enter__(self):
        self.tstart = time.time()
        return self

    def __exit__(self, type, value, traceback):
        self.elapsed = time.time() - self.tstart
        if self.verbose:
            if self.name:
                print('[%s]' % self.name),
            print ('Elapsed: %s' % (self.elapsed))



# -*- coding: utf-8 -*-
"""Nexus Operation : **SARA**

Calibration of the knee with the SARA method.

The script considers all frames of the c3d and detects the side autmaticallt from ANK marker trajectories

:param -s, --side [string]: lower limb side ( choice: Left or Right )
:param -b, --beginFrame [int]:  manual selection of the first  frame
:param -e, --endFrame [int]:   manual selection of the last  frame


Examples:
    In the script argument box of a python nexus operation, you can edit:

    >>>  -side=Left -b=50 -e=100
    (Left knee calibration between frames 50 and 100)
"""
import os
import logging
import argparse
import numpy as np

# pyCGM2 settings
import pyCGM2
from pyCGM2 import log; log.setLoggingLevel(logging.INFO)

# vicon nexus
try:
    import ViconNexus
except:
    from viconnexusapi import ViconNexus

# pyCGM2 libraries
from pyCGM2 import enums
from pyCGM2.Tools import btkTools
from pyCGM2.Utils import files
from pyCGM2.Nexus import nexusFilters, nexusUtils,nexusTools

from pyCGM2.Model import  modelFilters

from pyCGM2.Configurator import CgmArgsManager
from pyCGM2.Lib.CGM import  kneeCalibration



def main():

    parser = argparse.ArgumentParser(description='SARA Functional Knee Calibration')
    parser.add_argument('-s','--side', type=str, help="Side : Left or Right")
    parser.add_argument('-b','--beginFrame', type=int, help="begin frame")
    parser.add_argument('-e','--endFrame', type=int, help="end frame")
    args = parser.parse_args()


    NEXUS = ViconNexus.ViconNexus()
    NEXUS_PYTHON_CONNECTED = NEXUS.Client.IsConnected()


    if NEXUS_PYTHON_CONNECTED: # run Operation

        # --------------------------PATH + FILE ------------------------------------
        DATA_PATH, reconstructedFilenameLabelledNoExt = NEXUS.GetTrialName()

        reconstructFilenameLabelled = reconstructedFilenameLabelledNoExt+".c3d"

        logging.info( "data Path: "+ DATA_PATH )
        logging.info( "reconstructed file: "+ reconstructFilenameLabelled)

       # --------------------------SUBJECT -----------------------------------
        # Notice : Work with ONE subject by session
        subjects = NEXUS.GetSubjectNames()
        subject = nexusTools.getActiveSubject(NEXUS)
        logging.info(  "Subject name : " + subject  )

        # --------------------pyCGM2 MODEL ------------------------------
        model = files.loadModel(DATA_PATH,subject)

        logging.info("loaded model : %s" %(model.version ))
        # --------------------------CONFIG ------------------------------------

        # --------------------CHECKING ------------------------------
        if model.version in ["CGM1.0","CGM1.1","CGM2.1","CGM2.2"] :
            raise Exception ("Can t use SARA method with your model %s [minimal version : CGM2.3]"%(model.version))
        elif model.version == "CGM2.3":
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_3-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_3-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_3-pyCGM2.settings")
        elif model.version in  ["CGM2.4"]:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_4-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_4-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_4-pyCGM2.settings")
        elif model.version in  ["CGM2.5"]:
            if os.path.isfile(pyCGM2.PYCGM2_APPDATA_PATH + "CGM2_5-pyCGM2.settings"):
                settings = files.openFile(pyCGM2.PYCGM2_APPDATA_PATH,"CGM2_5-pyCGM2.settings")
            else:
                settings = files.openFile(pyCGM2.PYCGM2_SETTINGS_FOLDER,"CGM2_5-pyCGM2.settings")
        else:
            raise Exception ("model version not found [contact admin]")

        # --------------------------SESSION INFOS ------------------------------------
        mpInfo,mpFilename = files.getMpFileContent(DATA_PATH,"mp.pyCGM2",subject)

        #  translators management
        if model.version in  ["CGM2.3"]:
            translators = files.getTranslators(DATA_PATH,"CGM2-3.translators")
        elif model.version in  ["CGM2.4"]:
            translators = files.getTranslators(DATA_PATH,"CGM2-4.translators")
        elif model.version in  ["CGM2.5"]:
            translators = files.getTranslators(DATA_PATH,"CGM2-5.translators")
        if not translators:
           translators = settings["Translators"]

        # btkAcq builder
        nacf = nexusFilters.NexusConstructAcquisitionFilter(DATA_PATH,reconstructedFilenameLabelledNoExt,subject)
        acq = nacf.build()

        # --------------------------MODEL PROCESSING----------------------------
        model,acqFunc,side = kneeCalibration.sara(model,
            DATA_PATH,reconstructFilenameLabelled,translators,
            args.side,args.beginFrame,args.endFrame,forceBtkAcq=acq)

        # ----------------------SAVE-------------------------------------------
        files.saveModel(model,DATA_PATH,subject)
        logging.warning("model updated with a  %s knee calibrated with SARA method" %(side))

        # save mp
        files.saveMp(mpInfo,model,DATA_PATH,mpFilename)
        # ----------------------VICON INTERFACE-------------------------------------------
        #--- update mp
        nexusUtils.updateNexusSubjectMp(NEXUS,model,subject)


        # -- add nexus Bones
        if side == "Left":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"LFE0", model.getSegment("Left Thigh"),OriginValues = acqFunc.GetPoint("LKJC").GetValues() )
        elif side == "Right":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"RFE0", model.getSegment("Right Thigh"),OriginValues = acqFunc.GetPoint("RKJC").GetValues() )

        proximalSegmentLabel=str(side+" Thigh")
        distalSegmentLabel=str(side+" Shank")

        # add modelled markers
        meanOr_inThigh = model.getSegment(proximalSegmentLabel).getReferential("TF").getNodeTrajectory("KJC_Sara")
        meanAxis_inThigh = model.getSegment(proximalSegmentLabel).getReferential("TF").getNodeTrajectory("KJC_SaraAxis")
        btkTools.smartAppendPoint(acqFunc,side+"_KJC_Sara",meanOr_inThigh)
        btkTools.smartAppendPoint(acqFunc,side+"_KJC_SaraAxis",meanAxis_inThigh)

        nexusTools.appendModelledMarkerFromAcq(NEXUS,subject,side+"_KJC_Sara", acqFunc)
        nexusTools.appendModelledMarkerFromAcq(NEXUS,subject,side+"_KJC_SaraAxis", acqFunc)


        #---Second model motion filter

        # consider new anatomical frame
        scp=modelFilters.StaticCalibrationProcedure(model)
        modMotion=modelFilters.ModelMotionFilter(scp,acqFunc,model,enums.motionMethod.Sodervisk)
        modMotion.segmentalCompute([proximalSegmentLabel,distalSegmentLabel])


        # projection of the Sara axis in the transversale plane
        # -- add nexus Bones
        if side == "Left":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"LFE1", model.getSegment("Left Thigh"),OriginValues = acqFunc.GetPoint("LKJC").GetValues() )
            logging.warning("offset %s" %(str(model.mp_computed["LeftKneeFuncCalibrationOffset"] )))
        elif side == "Right":
            nexusTools.appendBones(NEXUS,subject,acqFunc,"RFE1", model.getSegment("Right Thigh"),OriginValues = acqFunc.GetPoint("RKJC").GetValues() )
            logging.warning("offset %s" %(str(model.mp_computed["RightKneeFuncCalibrationOffset"] )))


    else:
        raise Exception("NO Nexus connection. Turn on Nexus")

if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
import logging

# pyCGM2
from pyCGM2.Tools import btkTools


class C3dManager(object):
    def __init__ (self):
        self.spatioTemporal={"Acqs":None , "Filenames":None}
        self.kinematic={"Acqs":None , "Filenames":None}
        self.kineticFlag = False
        self.kinetic={"Acqs": None, "Filenames":None }
        self.emg={"Acqs":None , "Filenames":None}



class UniqueBtkAcqSetProcedure(object):


    def __init__(self, data_path, fileLst,acqs):
        self.m_files = fileLst
        self.m_data_path = data_path
        self.m_acqs = acqs



    def generate(self,c3dManager,spatioTempFlag,kinematicFlag,kineticFlag,emgFlag):


        #---spatioTemporalTrials
        if spatioTempFlag:
            c3dManager.spatioTemporal["Acqs"] = self.m_acqs
            c3dManager.spatioTemporal["Filenames"] = self.m_files

        # ----kinematic trials---
        if kinematicFlag:
            c3dManager.kinematic["Acqs"] = self.m_acqs
            c3dManager.kinematic["Filenames"] = self.m_files

        #---kinetic Trials--- ( check if kinetic events)
        if kineticFlag:
            c3dManager.kinetic["Acqs"],c3dManager.kinetic["Filenames"],C3dManager.kineticFlag =  btkTools.automaticKineticDetection(self.m_data_path,self.m_files,acqs = self.m_acqs)

        #----emgTrials
        if emgFlag:
            c3dManager.emg["Acqs"] =self.m_acqs
            c3dManager.emg["Filenames"] = self.m_files



class UniqueC3dSetProcedure(object):


    def __init__(self, data_path, fileLst):
        self.m_files = fileLst
        self.m_data_path = data_path



    def generate(self,c3dManager,spatioTempFlag,kinematicFlag,kineticFlag,emgFlag):


        #---spatioTemporalTrials
        if spatioTempFlag:
            c3dManager.spatioTemporal["Acqs"],c3dManager.spatioTemporal["Filenames"] = btkTools.buildTrials(self.m_data_path,self.m_files)


        # ----kinematic trials---
        if kinematicFlag:
            c3dManager.kinematic["Acqs"],c3dManager.kinematic["Filenames"], = btkTools.buildTrials(self.m_data_path,self.m_files)

        #---kinetic Trials--- ( check if kinetic events)
        if kineticFlag:
            c3dManager.kinetic["Acqs"],c3dManager.kinetic["Filenames"],C3dManager.kineticFlag =  btkTools.automaticKineticDetection(self.m_data_path,self.m_files)


        #----emgTrials
        if emgFlag:
            c3dManager.emg["Acqs"],c3dManager.emg["Filenames"], = btkTools.buildTrials(self.m_data_path,self.m_files)


class DistinctC3dSetProcedure(object):


    def __init__(self, data_path, stp_fileLst, kinematic_fileLst, kinetic_fileLst, emg_fileLst):

        self.m_data_path = data_path

        self.m_files_stp = stp_fileLst
        self.m_files_kinematic = kinematic_fileLst
        self.m_files_kinetic = kinetic_fileLst
        self.m_files_emg = emg_fileLst

    def generate(self,c3dManager,spatioTempFlag,kinematicFlag,kineticFlag,emgFlag):


        #---spatioTemporalTrials
        if spatioTempFlag:
            c3dManager.spatioTemporal["Acqs"],c3dManager.spatioTemporal["Filenames"] = btkTools.buildTrials(self.m_data_path,self.m_files_stp)


        # ----kinematic trials---
        if kinematicFlag:
            c3dManager.kinematic["Acqs"],c3dManager.kinematic["Filenames"], = btkTools.buildTrials(self.m_data_path,self.m_files_kinematic)

        #---kinetic Trials--- ( check if kinetic events)
        if kineticFlag:
            c3dManager.kinetic["Acqs"],c3dManager.kinetic["Filenames"],C3dManager.kineticFlag =  btkTools.automaticKineticDetection(self.m_data_path,self.m_files_kinetic)


        #----emgTrials
        if emgFlag:
            c3dManager.emg["Acqs"],c3dManager.emg["Filenames"], = btkTools.buildTrials(self.m_data_path,self.m_files_emg)




class C3dManagerFilter(object):
    """

    """

    def __init__(self,procedure):

        self.m_procedure = procedure
        self.m_spatioTempFlag = True
        self.m_kinematicFlag = True
        self.m_kineticFlag = True
        self.m_emgFlag = True

    def enableSpatioTemporal(self, boolean):
        self.m_spatioTempFlag = boolean

    def enableKinematic(self, boolean):
        self.m_kinematicFlag = boolean

    def enableKinetic(self, boolean):
        self.m_kineticFlag = boolean

    def enableEmg(self, boolean):
        self.m_emgFlag = boolean



    def generate(self):

        c3dManager = C3dManager()


        self.m_procedure.generate(c3dManager,self.m_spatioTempFlag, self.m_kinematicFlag, self.m_kineticFlag, self.m_emgFlag)

        return c3dManager
